@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_STDLIB_H
@@ -27,9 +27,9 @@
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunar/thunar-column-model.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-column-model.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
 
 
 
