@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_MEMORY_H
@@ -34,15 +34,15 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-icon-factory.h>
-#include <thunar/thunar-io-jobs.h>
-#include <thunar/thunar-job.h>
-#include <thunar/thunar-pango-extensions.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-util.h>
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-icon-factory.h"
+#include "thunar/thunar-io-jobs.h"
+#include "thunar/thunar-job.h"
+#include "thunar/thunar-pango-extensions.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-util.h"
 
 
 
@@ -222,6 +222,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
 {
   ThunarIconFactory *icon_factory;
   GtkIconTheme      *icon_theme;
+  gint               scale_factor;
   const gchar       *filename;
   const gchar       *text;
   ThunarJob         *job = NULL;
@@ -231,6 +232,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   GtkWidget         *grid;
   GtkWindow         *window;
   GdkPixbuf         *icon;
+  cairo_surface_t   *surface;
   GdkScreen         *screen;
   gchar             *title;
   gint               response;
@@ -248,7 +250,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   screen = thunar_util_parse_parent (parent, &window);
 
   /* get the filename of the file */
-  filename = thunar_file_get_display_name (file);
+  filename = thunar_file_get_basename (file);
 
   /* create a new dialog window */
   title = g_strdup_printf (_("Rename \"%s\""), filename);
@@ -266,6 +268,11 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   if (G_UNLIKELY (window == NULL && screen != NULL))
     gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
+  if (window != NULL)
+    scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (window));
+  else
+    scale_factor = gtk_widget_get_scale_factor (dialog);
+
   grid = gtk_grid_new ();
   gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
   gtk_grid_set_row_spacing (GTK_GRID (grid), 3);
@@ -275,10 +282,11 @@ thunar_dialogs_show_rename_file (gpointer               parent,
 
   icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (dialog));
   icon_factory = thunar_icon_factory_get_for_icon_theme (icon_theme);
-  icon = thunar_icon_factory_load_file_icon (icon_factory, file, THUNAR_FILE_ICON_STATE_DEFAULT, 48);
+  icon = thunar_icon_factory_load_file_icon (icon_factory, file, THUNAR_FILE_ICON_STATE_DEFAULT, 48, scale_factor);
+  surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, NULL);
   g_object_unref (G_OBJECT (icon_factory));
 
-  image = gtk_image_new_from_pixbuf (icon);
+  image = gtk_image_new_from_surface (surface);
   gtk_widget_set_margin_start (GTK_WIDGET (image), 6);
   gtk_widget_set_margin_end (GTK_WIDGET (image), 6);
   gtk_widget_set_margin_top (GTK_WIDGET (image), 6);
@@ -286,6 +294,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   gtk_widget_set_valign (GTK_WIDGET (image), GTK_ALIGN_START);
   gtk_grid_attach (GTK_GRID (grid), image, 0, row, 1, 2);
   g_object_unref (G_OBJECT (icon));
+  cairo_surface_destroy (surface);
   gtk_widget_show (image);
 
   label = gtk_label_new (_("Enter the new name:"));
@@ -758,6 +767,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   GtkWidget         *renameall_button;
   GtkWidget         *rename_button;
   GdkPixbuf         *icon;
+  cairo_surface_t   *surface;
   gchar             *date_custom_style;
   gchar             *date_string;
   gchar             *size_string;
@@ -765,6 +775,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   gint               response;
   gboolean           file_size_binary;
   gint               row = 0;
+  gint               scale_factor;
 
   _thunar_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), THUNAR_JOB_RESPONSE_CANCEL);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (src_file), THUNAR_JOB_RESPONSE_CANCEL);
@@ -786,6 +797,11 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_JOB_RESPONSE_REPLACE);
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
+  if (parent != NULL)
+    scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (parent));
+  else
+    scale_factor = gtk_widget_get_scale_factor (dialog);
+
   /* determine the icon factory to use */
   icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (dialog));
   icon_factory = thunar_icon_factory_get_for_icon_theme (icon_theme);
@@ -859,8 +875,13 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
     }
   else
     {
-      text = g_strdup_printf (_("This folder already contains a file \"%s\"."),
-                              thunar_file_get_display_name (dst_file));
+      if (g_strcmp0 (thunar_file_get_display_name (dst_file), thunar_file_get_basename (dst_file)) != 0)
+        text = g_strdup_printf (_("This folder already contains a file \"%s\" (%s)."),
+                                thunar_file_get_display_name (dst_file),
+                                thunar_file_get_basename (dst_file));
+      else
+        text = g_strdup_printf (_("This folder already contains a file \"%s\"."),
+                                thunar_file_get_display_name (dst_file));
     }
 
   label = gtk_label_new (text);
@@ -895,14 +916,16 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   /* next row */
   row++;
 
-  icon = thunar_icon_factory_load_file_icon (icon_factory, dst_file, THUNAR_FILE_ICON_STATE_DEFAULT, 48);
-  image = gtk_image_new_from_pixbuf (icon);
+  icon = thunar_icon_factory_load_file_icon (icon_factory, dst_file, THUNAR_FILE_ICON_STATE_DEFAULT, 48, scale_factor);
+  surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, NULL);
+  image = gtk_image_new_from_surface (surface);
   gtk_widget_set_margin_start (GTK_WIDGET(image), 6);
   gtk_widget_set_margin_end (GTK_WIDGET(image), 6);
   gtk_widget_set_margin_top (GTK_WIDGET(image), 6);
   gtk_widget_set_margin_bottom (GTK_WIDGET(image), 6);
   gtk_grid_attach (GTK_GRID (grid), image, 1, row, 1, 1);
   g_object_unref (G_OBJECT (icon));
+  cairo_surface_destroy (surface);
   gtk_widget_show (image);
 
   size_string = thunar_file_get_size_string_long (dst_file, file_size_binary);
@@ -940,14 +963,16 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   /* next row */
   row++;
 
-  icon = thunar_icon_factory_load_file_icon (icon_factory, src_file, THUNAR_FILE_ICON_STATE_DEFAULT, 48);
-  image = gtk_image_new_from_pixbuf (icon);
+  icon = thunar_icon_factory_load_file_icon (icon_factory, src_file, THUNAR_FILE_ICON_STATE_DEFAULT, 48, scale_factor);
+  surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, NULL);
+  image = gtk_image_new_from_surface (surface);
   gtk_widget_set_margin_start (GTK_WIDGET(image), 6);
   gtk_widget_set_margin_end (GTK_WIDGET(image), 6);
   gtk_widget_set_margin_top (GTK_WIDGET(image), 6);
   gtk_widget_set_margin_bottom (GTK_WIDGET(image), 6);
   gtk_grid_attach (GTK_GRID (grid), image, 1, row, 1, 1);
   g_object_unref (G_OBJECT (icon));
+  cairo_surface_destroy (surface);
   gtk_widget_show (image);
 
   size_string = thunar_file_get_size_string_long (src_file, file_size_binary);
@@ -1053,6 +1078,8 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
   GtkWidget      *dialog;
   GString        *secondary;
   GError         *err = NULL;
+  gchar          *file_name;
+  gchar          *executable;
 
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
   _thunar_return_val_if_fail (g_utf8_validate (command, -1, NULL), FALSE);
@@ -1060,13 +1087,29 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
   /* parse the parent window and screen */
   screen = thunar_util_parse_parent (parent, &window);
 
-  /* secondary text */
+  /* create the secondary text */
+  if (g_strcmp0 (thunar_file_get_display_name (file), thunar_file_get_basename (file)) != 0)
+    file_name = g_strdup_printf ("\"%s\" (%s)", thunar_file_get_display_name (file), thunar_file_get_basename (file));
+  else
+    file_name = g_strdup_printf ("\"%s\"", thunar_file_get_display_name (file));
+    
+  if (g_file_info_get_attribute_boolean (thunar_file_get_info (file), G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE) == FALSE)
+    /* TRANSLATORS: This string will be inserted in the following sentence in the second appearance of %s:
+     * "The desktop file %s is in an insecure location and not marked as secure%s." */
+    executable = g_strdup (_("/executable"));
+  else
+    executable = g_strdup ("");
+    
   secondary = g_string_new (NULL);
-  g_string_append_printf (secondary, _("The desktop file \"%s\" is in an insecure location "
-                                       "and not marked as executable. If you do not trust "
-                                       "this program, click Cancel."),
-                                       thunar_file_get_display_name (file));
-  g_string_append (secondary, "\n\n");
+  g_string_printf (secondary,
+                   _("The desktop file %s is in an insecure location and not marked as secure%s. "
+                     "If you do not trust this program, click Cancel.\n\n"),
+                   file_name,
+                   executable);
+  
+  g_free (file_name);
+  g_free (executable);
+  
   if (g_uri_is_valid (command, G_URI_FLAGS_NONE, NULL))
     g_string_append_printf (secondary, G_KEY_FILE_DESKTOP_KEY_URL"=%s", command);
   else
@@ -1082,7 +1125,7 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
   gtk_window_set_title (GTK_WINDOW (dialog), _("Attention"));
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Launch Anyway"), GTK_RESPONSE_OK);
   if (thunar_file_is_chmodable (file))
-    gtk_dialog_add_button (GTK_DIALOG (dialog), _("Mark _Executable"), GTK_RESPONSE_APPLY);
+    gtk_dialog_add_button (GTK_DIALOG (dialog), _("Mark As _Secure And Launch"), GTK_RESPONSE_APPLY);
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
   if (screen != NULL && window == NULL)
@@ -1142,3 +1185,232 @@ static void thunar_dialogs_select_filename (GtkWidget *entry)
   if (G_LIKELY (offset > 0))
     gtk_editable_select_region (GTK_EDITABLE (entry), 0, offset);
 }
+
+
+
+/**
+ * thunar_dialog_confirm_close_split_pane_tabs:
+ * @parent              : (allow-none): transient parent of the dialog, or %NULL.
+ *
+ * Runs a dialog to ask the user whether they want to close a split pane with multiple tabs
+ *
+ * Return value: #GTK_RESPONSE_CANCEL if cancelled, #GTK_RESPONSE_CLOSE if the user
+ * wants to close all tabs of the other split pane
+ */
+gint
+thunar_dialog_confirm_close_split_pane_tabs (GtkWindow *parent)
+{
+  GtkWidget         *dialog, *checkbutton, *vbox;
+  const gchar       *primary_text, *warning_icon;
+  gchar             *secondary_text;
+  ThunarPreferences *preferences;
+  gint               response;
+
+  g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), GTK_RESPONSE_NONE);
+
+  primary_text = _("Close split pane with multiple tabs?");
+  secondary_text = g_strdup (_("The other split pane has multiple open tabs. Closing it "
+                                "will close all these tabs."));
+
+  warning_icon = "dialog-warning";
+
+  dialog = xfce_message_dialog_new (parent,
+                                    _("Warning"),
+                                    warning_icon,
+                                    primary_text,
+                                    secondary_text,
+                                    XFCE_BUTTON_TYPE_MIXED, NULL, _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                    XFCE_BUTTON_TYPE_MIXED, NULL, _("Close Pane"), GTK_RESPONSE_CLOSE,
+                                    NULL);
+
+  checkbutton = gtk_check_button_new_with_mnemonic (_("Do _not ask me again"));
+  vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  gtk_box_pack_end (GTK_BOX (vbox), checkbutton, FALSE, FALSE, 5);
+  g_object_set (G_OBJECT (checkbutton), "halign", GTK_ALIGN_END, "margin-start", 6, "margin-end", 6, NULL);
+  gtk_widget_set_hexpand (checkbutton, TRUE);
+
+  gtk_widget_show_all (dialog);
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  /* if the user requested not to be asked again, store this preference */
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)))
+    {
+      preferences = thunar_preferences_get ();
+      g_object_set (G_OBJECT (preferences), "misc-confirm-close-multiple-tabs", FALSE, NULL);
+      g_object_unref (G_OBJECT (preferences));
+    }
+    
+  gtk_widget_destroy (dialog);
+  g_free (secondary_text);
+  return response;
+}
+
+
+
+/**
+ * thunar_dialog_show_rename_launcher_options:
+ * @parent : a #GtkWidget on which the error dialog should be shown, or a #GdkScreen
+ *           if no #GtkWidget is known. May also be %NULL, in which case the default
+ *           #GdkScreen will be used.
+ *
+ * Shows a dialog where the user is asked if he wants to edit the launcher name or the actual filename.
+ *
+ * Return value: THUNAR_RESPONSE_FILENAME if the user chooses "Filename", THUNAR_RESPONSE_LAUNCHERNAME
+ * if the user selects "Launchername" and GTK_RESPONSE_DELETE_EVENT if the user deletes the dialog.
+ **/
+gint
+thunar_dialog_show_rename_launcher_options (GtkWindow *parent)
+{
+  gint       response;
+  GtkWidget *dialog;
+  GtkWidget *button;
+  
+  dialog = gtk_message_dialog_new (parent,
+                                   GTK_DIALOG_MODAL |
+                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_QUESTION,
+                                   GTK_BUTTONS_NONE,
+                                   "%s", _("Do you want to rename the launcher, or the file itself?"));
+  
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Rename Launcher Options"));
+  
+  button = gtk_button_new_with_mnemonic (_("Rename _File"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, THUNAR_RESPONSE_FILENAME);
+  gtk_widget_show (button);
+  
+  button = gtk_button_new_with_mnemonic (_("Rename _Launcher"));
+  gtk_widget_set_can_default (button, TRUE);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, THUNAR_RESPONSE_LAUNCHERNAME);
+  gtk_widget_show (button);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_RESPONSE_LAUNCHERNAME);
+  
+  /* run the question dialog */
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+
+  return response;
+}
+
+
+
+/**
+ * thunar_dialog_ask_execute:
+ * @file        : a #ThunarFile pointer
+ * @parent      : a #GtkWidget on which the error dialog should be shown, or a #GdkScreen
+ *                if no #GtkWidget is known. May also be %NULL, in which case the default
+ *                #GdkScreen will be used.
+ * @allow_open  : if set, the "Open" button is visible.
+ * @single_file : if set, the file name is visible in dialog message. Otherwise there is
+ *                generic message about selected files.
+ *
+ * Shows a dialog where the user is asked if and how he wants to run the executable script.
+ *
+ * Return value: One of #ThunarFileAskExecuteResponse enum value.
+ **/
+gint
+thunar_dialog_ask_execute (const ThunarFile *file,
+                           gpointer          parent,
+                           gboolean          allow_open,
+                           gboolean          single_file)
+{
+  GtkWidget *dialog;
+  GtkWindow *window;
+  GdkScreen *screen;
+  gint       response;
+  gchar     *dialog_text;
+  GtkWidget *button;
+
+  _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
+  _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent), THUNAR_FILE_ASK_EXECUTE_RESPONSE_OPEN);
+
+  /* parse the parent window and screen */
+  screen = thunar_util_parse_parent (parent, &window);
+
+  if (single_file)
+    {
+      gchar *basename = g_file_get_basename (thunar_file_get_file (file));
+      dialog_text = g_strdup_printf (_("The file \"%s\" seems to be executable. What do you want to do with it?"), basename);
+      g_free (basename);
+    }
+  else
+    {
+      dialog_text = g_strdup (_("The selected files seem to be executable. What do you want to do with them?"));
+    }
+
+  dialog = gtk_message_dialog_new (window,
+                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_QUESTION,
+                                   GTK_BUTTONS_NONE,
+                                   "%s", dialog_text);
+
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Open Shell Script"));
+
+  g_free (dialog_text);
+
+  button = gtk_button_new_with_mnemonic (_("Run In _Terminal"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, THUNAR_FILE_ASK_EXECUTE_RESPONSE_RUN_IN_TERMINAL);
+  gtk_widget_show (button);
+
+  button = gtk_button_new_with_mnemonic (_("_Execute"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, THUNAR_FILE_ASK_EXECUTE_RESPONSE_RUN);
+  gtk_widget_show (button);
+
+  if (allow_open)
+    {
+      button = gtk_button_new_with_mnemonic (_("_Open"));
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, THUNAR_FILE_ASK_EXECUTE_RESPONSE_OPEN);
+      gtk_widget_show (button);
+
+      gtk_widget_set_can_default (button, TRUE);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_FILE_ASK_EXECUTE_RESPONSE_OPEN);
+    }
+  else
+    {
+      gtk_widget_set_can_default (button, TRUE);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_FILE_ASK_EXECUTE_RESPONSE_RUN);
+    }
+
+  if (G_UNLIKELY (window == NULL && screen != NULL))
+    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+
+  return response;
+}
+
+
+
+/**
+ * thunar_dialog_show_launcher_props:
+ * @launcher   : a #ThunarFile.
+ * @parent     : a #GdkScreen, a #GtkWidget or %NULL. If %NULL is passed then the default screen will be used.
+ *
+ * Calls the exo-desktop-item-edit command to edit the properties of a .desktop file.
+ * A dialog with the current properties will appear.
+ **/
+void
+thunar_dialog_show_launcher_props (ThunarFile *launcher,
+                                   gpointer    parent)
+{
+  const gchar *display_name;
+  gchar       *cmd   = NULL,
+              *uri   = NULL;
+  GError      *error = NULL;
+  GdkScreen   *screen;
+
+  _thunar_return_if_fail (launcher == NULL || THUNAR_IS_FILE (launcher));
+  _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
+  
+  screen = thunar_util_parse_parent (parent, NULL);
+  uri = thunar_file_dup_uri (launcher);
+  display_name = gdk_display_get_name (gdk_screen_get_display (screen));
+  cmd = g_strdup_printf ("exo-desktop-item-edit \"--display=%s\" \"%s\"", display_name, uri);
+
+  if (xfce_spawn_command_line (NULL, cmd, FALSE, FALSE, FALSE, &error) == FALSE)
+    thunar_dialogs_show_error (screen, error, _("Failed to edit launcher via command \"%s\""), cmd);
+
+  g_free (cmd);
+  g_free (uri);
+  g_clear_error (&error);
+}
