@@ -20,9 +20,9 @@
 #ifndef __THUNAR_DIALOGS_H__
 #define __THUNAR_DIALOGS_H__
 
-#include <thunar/thunar-enum-types.h>
-#include <thunar/thunar-file.h>
-#include <thunar/thunar-job.h>
+#include "thunar/thunar-enum-types.h"
+#include "thunar/thunar-file.h"
+#include "thunar/thunar-job.h"
 
 G_BEGIN_DECLS;
 
@@ -53,6 +53,14 @@ gboolean           thunar_dialogs_show_insecure_program (gpointer              p
                                                          const gchar          *title,
                                                          ThunarFile           *file,
                                                          const gchar          *command);
+gint               thunar_dialog_confirm_close_split_pane_tabs (GtkWindow     *parent);
+gint               thunar_dialog_show_rename_launcher_options  (GtkWindow     *parent);
+gint               thunar_dialog_ask_execute            (const ThunarFile     *file,
+                                                         gpointer              parent,
+                                                         gboolean              allow_open,
+                                                         gboolean              single_file);
+void               thunar_dialog_show_launcher_props    (ThunarFile           *launcher,
+                                                         gpointer              parent);
 
 G_END_DECLS;
 
