@@ -19,17 +19,15 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#include <libxfce4util/libxfce4util.h>
-
-#include <thunar/thunar-file-monitor.h>
-#include <thunar/thunar-folder.h>
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-io-jobs.h>
-#include <thunar/thunar-job.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-file-monitor.h"
+#include "thunar/thunar-folder.h"
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-io-jobs.h"
+#include "thunar/thunar-job.h"
+#include "thunar/thunar-private.h"
 
 #define DEBUG_FILE_CHANGES FALSE
 
@@ -55,36 +53,38 @@ enum
 
 
 
-static void     thunar_folder_dispose                     (GObject                *object);
-static void     thunar_folder_finalize                    (GObject                *object);
-static void     thunar_folder_get_property                (GObject                *object,
-                                                           guint                   prop_id,
-                                                           GValue                 *value,
-                                                           GParamSpec             *pspec);
-static void     thunar_folder_set_property                (GObject                *object,
-                                                           guint                   prop_uid,
-                                                           const GValue           *value,
-                                                           GParamSpec             *pspec);
-static void     thunar_folder_real_destroy                (ThunarFolder           *folder);
-static void     thunar_folder_error                       (ExoJob                 *job,
-                                                           GError                 *error,
-                                                           ThunarFolder           *folder);
-static gboolean thunar_folder_files_ready                 (ThunarJob              *job,
-                                                           GList                  *files,
-                                                           ThunarFolder           *folder);
-static void     thunar_folder_finished                    (ExoJob                 *job,
-                                                           ThunarFolder           *folder);
-static void     thunar_folder_file_changed                (ThunarFileMonitor      *file_monitor,
-                                                           ThunarFile             *file,
-                                                           ThunarFolder           *folder);
-static void     thunar_folder_file_destroyed              (ThunarFileMonitor      *file_monitor,
-                                                           ThunarFile             *file,
-                                                           ThunarFolder           *folder);
-static void     thunar_folder_monitor                     (GFileMonitor           *monitor,
-                                                           GFile                  *file,
-                                                           GFile                  *other_file,
-                                                           GFileMonitorEvent       event_type,
-                                                           gpointer                user_data);
+static void           thunar_folder_dispose               (GObject               *object);
+static void           thunar_folder_finalize              (GObject               *object);
+static void           thunar_folder_get_property          (GObject               *object,
+                                                           guint                  prop_id,
+                                                           GValue                *value,
+                                                           GParamSpec            *pspec);
+static void           thunar_folder_set_property          (GObject               *object,
+                                                           guint                  prop_uid,
+                                                           const GValue          *value,
+                                                           GParamSpec            *pspec);
+static void           thunar_folder_real_destroy          (ThunarFolder          *folder);
+static void           thunar_folder_error                 (ExoJob                *job,
+                                                           GError                *error,
+                                                           ThunarFolder          *folder);
+static gboolean       thunar_folder_files_ready           (ThunarJob             *job,
+                                                           GList                 *files,
+                                                           ThunarFolder          *folder);
+static void           thunar_folder_finished              (ExoJob                *job,
+                                                           ThunarFolder          *folder);
+static void           thunar_folder_file_changed          (ThunarFileMonitor     *file_monitor,
+                                                           ThunarFile            *file,
+                                                           ThunarFolder          *folder);
+static void           thunar_folder_file_destroyed        (ThunarFileMonitor     *file_monitor,
+                                                           ThunarFile            *file,
+                                                           ThunarFolder          *folder);
+static void           thunar_folder_monitor               (GFileMonitor          *monitor,
+                                                           GFile                 *file,
+                                                           GFile                 *other_file,
+                                                           GFileMonitorEvent      event_type,
+                                                           gpointer               user_data);
+static void           thunar_folder_load_content_types    (ThunarFolder          *folder,
+                                                           GList                 *files);
 
 
 
@@ -104,17 +104,20 @@ struct _ThunarFolderClass
 
 struct _ThunarFolder
 {
-  GObject __parent__;
+  GObject            __parent__;
 
   ThunarJob         *job;
+  ThunarJob         *content_type_job;
 
   ThunarFile        *corresponding_file;
-  GList             *new_files;
-  GList             *files;
-  gboolean           reload_info;
 
-  GList             *content_type_ptr;
-  guint              content_type_idle_id;
+  /* map of GFile --> ThunarFile */
+  GHashTable        *new_files_map;
+
+  /* map of GFile --> ThunarFile */
+  GHashTable        *files_map;
+
+  gboolean           reload_info;
 
   guint              in_destruction : 1;
 
@@ -138,13 +141,13 @@ static void
 thunar_folder_constructed (GObject *object)
 {
   ThunarFolder *folder = THUNAR_FOLDER (object);
-  GError       *error  = NULL;
+  GError       *error = NULL;
 
   folder->monitor = g_file_monitor_directory (thunar_file_get_file (folder->corresponding_file),
                                               G_FILE_MONITOR_WATCH_MOVES, NULL, &error);
 
   if (G_LIKELY (folder->monitor != NULL))
-      g_signal_connect (folder->monitor, "changed", G_CALLBACK (thunar_folder_monitor), folder);
+    g_signal_connect (folder->monitor, "changed", G_CALLBACK (thunar_folder_monitor), folder);
   else
     {
       g_debug ("Could not create folder monitor: %s", error->message);
@@ -182,8 +185,8 @@ thunar_folder_class_init (ThunarFolderClass *klass)
                                                         "corresponding-file",
                                                         THUNAR_TYPE_FILE,
                                                         G_PARAM_READABLE
-                                                        | G_PARAM_WRITABLE
-                                                        | G_PARAM_CONSTRUCT_ONLY));
+                                                          | G_PARAM_WRITABLE
+                                                          | G_PARAM_CONSTRUCT_ONLY));
 
   /**
    * ThunarFolder::loading:
@@ -205,7 +208,7 @@ thunar_folder_class_init (ThunarFolderClass *klass)
    * Emitted when the #ThunarFolder is destroyed.
    **/
   folder_signals[DESTROY] =
-    g_signal_new (I_("destroy"),
+    g_signal_new (I_ ("destroy"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
                   G_STRUCT_OFFSET (ThunarFolderClass, destroy),
@@ -222,7 +225,7 @@ thunar_folder_class_init (ThunarFolderClass *klass)
    * load the directory content because of an error.
    **/
   folder_signals[ERROR] =
-    g_signal_new (I_("error"),
+    g_signal_new (I_ ("error"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ThunarFolderClass, error),
@@ -237,7 +240,7 @@ thunar_folder_class_init (ThunarFolderClass *klass)
    * been added to a particular folder.
    **/
   folder_signals[FILES_ADDED] =
-    g_signal_new (I_("files-added"),
+    g_signal_new (I_ ("files-added"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ThunarFolderClass, files_added),
@@ -254,7 +257,7 @@ thunar_folder_class_init (ThunarFolderClass *klass)
    * the reload of folders, which take longer to load.
    **/
   folder_signals[FILES_REMOVED] =
-    g_signal_new (I_("files-removed"),
+    g_signal_new (I_ ("files-removed"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ThunarFolderClass, files_removed),
@@ -268,6 +271,9 @@ thunar_folder_class_init (ThunarFolderClass *klass)
 static void
 thunar_folder_init (ThunarFolder *folder)
 {
+  folder->files_map = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, NULL, g_object_unref);
+  folder->new_files_map = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, NULL, g_object_unref);
+
   /* connect to the ThunarFileMonitor instance */
   folder->file_monitor = thunar_file_monitor_get_default ();
   g_signal_connect (G_OBJECT (folder->file_monitor), "file-changed", G_CALLBACK (thunar_folder_file_changed), folder);
@@ -332,15 +338,9 @@ thunar_folder_finalize (GObject *object)
       g_object_unref (G_OBJECT (folder->corresponding_file));
     }
 
-  /* stop metadata collector */
-  if (folder->content_type_idle_id != 0)
-    g_source_remove (folder->content_type_idle_id);
-
-  /* release references to the new files */
-  thunar_g_list_free_full (folder->new_files);
-
-  /* release references to the current files */
-  thunar_g_list_free_full (folder->files);
+  /* release references to the current files lists */
+  g_hash_table_destroy (folder->files_map);
+  g_hash_table_destroy (folder->new_files_map);
 
   (*G_OBJECT_CLASS (thunar_folder_parent_class)->finalize) (object);
 }
@@ -357,17 +357,17 @@ thunar_folder_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_CORRESPONDING_FILE:
-      g_value_set_object (value, folder->corresponding_file);
-      break;
+      case PROP_CORRESPONDING_FILE:
+        g_value_set_object (value, folder->corresponding_file);
+        break;
 
-    case PROP_LOADING:
-      g_value_set_boolean (value, thunar_folder_get_loading (folder));
-      break;
+      case PROP_LOADING:
+        g_value_set_boolean (value, thunar_folder_get_loading (folder));
+        break;
 
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
     }
 }
 
@@ -383,19 +383,19 @@ thunar_folder_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_CORRESPONDING_FILE:
-      folder->corresponding_file = g_value_dup_object (value);
-      if (folder->corresponding_file)
-        thunar_file_watch (folder->corresponding_file);
-      break;
+      case PROP_CORRESPONDING_FILE:
+        folder->corresponding_file = g_value_dup_object (value);
+        if (folder->corresponding_file)
+          thunar_file_watch (folder->corresponding_file);
+        break;
 
-    case PROP_LOADING:
-      _thunar_assert_not_reached ();
-      break;
+      case PROP_LOADING:
+        _thunar_assert_not_reached ();
+        break;
 
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
     }
 }
 
@@ -428,11 +428,16 @@ thunar_folder_files_ready (ThunarJob    *job,
                            GList        *files,
                            ThunarFolder *folder)
 {
+  GList *lp;
+
   _thunar_return_val_if_fail (THUNAR_IS_FOLDER (folder), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
 
   /* merge the list with the existing list of new files */
-  folder->new_files = g_list_concat (folder->new_files, files);
+  for (lp = files; lp != NULL; lp = lp->next)
+    g_hash_table_insert (folder->new_files_map, thunar_file_get_file (lp->data), g_object_ref (lp->data));
+
+  thunar_g_list_free_full (files);
 
   /* indicate that we took over ownership of the file list */
   return TRUE;
@@ -440,58 +445,30 @@ thunar_folder_files_ready (ThunarJob    *job,
 
 
 
-static gboolean
-thunar_folder_content_type_loader_idle (gpointer data)
-{
-  ThunarFolder *folder;
-  GList        *lp;
-
-  _thunar_return_val_if_fail (THUNAR_IS_FOLDER (data), FALSE);
-
-  folder = THUNAR_FOLDER (data);
-
-  /* load another files content type */
-  for (lp = folder->content_type_ptr; lp != NULL; lp = lp->next)
-    if (thunar_file_load_content_type (lp->data))
-      {
-        /* if this was the last file, abort */
-        if (G_UNLIKELY (lp->next == NULL))
-          break;
-
-        /* set pointer to next file for the next iteration */
-        folder->content_type_ptr = lp->next;
-
-        return TRUE;
-      }
-
-  /* all content types loaded */
-  return FALSE;
-}
-
-
-
-static void
-thunar_folder_content_type_loader_idle_destroyed (gpointer data)
-{
-  _thunar_return_if_fail (THUNAR_IS_FOLDER (data));
-
-  THUNAR_FOLDER (data)->content_type_idle_id = 0;
-}
-
-
-
-static void
-thunar_folder_content_type_loader (ThunarFolder *folder)
+/**
+ * thunar_folder_load_content_types:
+ * @folder : a #ThunarFolder instance.
+ * @files : a #GList of #ThunarFile's for which the content type needs to be loaded.
+ *
+ * Starts a job to load the content type of all files inside the folder
+ **/
+void
+thunar_folder_load_content_types (ThunarFolder *folder,
+                                  GList        *files)
 {
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
-  _thunar_return_if_fail (folder->content_type_idle_id == 0);
 
-  /* set the pointer to the start of the list */
-  folder->content_type_ptr = folder->files;
+  /* check if we are currently connect to a job */
+  if (G_UNLIKELY (folder->content_type_job != NULL))
+    {
+      exo_job_cancel (EXO_JOB (folder->content_type_job));
+      g_object_unref (folder->content_type_job);
+      folder->content_type_job = NULL;
+    }
 
-  /* schedule idle */
-  folder->content_type_idle_id = g_idle_add_full (G_PRIORITY_LOW, thunar_folder_content_type_loader_idle,
-                                                  folder, thunar_folder_content_type_loader_idle_destroyed);
+  /* start a new content_type_job */
+  folder->content_type_job = thunar_io_jobs_load_content_types (files);
+  exo_job_launch (EXO_JOB (folder->content_type_job));
 }
 
 
@@ -500,98 +477,104 @@ static void
 thunar_folder_finished (ExoJob       *job,
                         ThunarFolder *folder)
 {
-  ThunarFile *file;
-  GList      *files;
-  GList      *lp;
+  ThunarFile     *file;
+  GHashTableIter iter;
+  gpointer       key, value;
+  GList          *files = NULL;
+  GList          *lp;
 
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
   _thunar_return_if_fail (THUNAR_IS_JOB (job));
   _thunar_return_if_fail (THUNAR_IS_FILE (folder->corresponding_file));
-  _thunar_return_if_fail (folder->content_type_idle_id == 0);
 
-  /* check if we need to merge new files with existing files */
-  if (G_UNLIKELY (folder->files != NULL))
+  /* determine all added files (files on new_files, but not on files) */
+  g_hash_table_iter_init (&iter, folder->new_files_map);
+  while (g_hash_table_iter_next (&iter, &key, &value))
     {
-      /* determine all added files (files on new_files, but not on files) */
-      for (files = NULL, lp = folder->new_files; lp != NULL; lp = lp->next)
-        if (g_list_find (folder->files, lp->data) == NULL)
-          {
-            /* put the file on the added list */
-            files = g_list_prepend (files, lp->data);
-
-            /* add to the internal files list */
-            folder->files = g_list_prepend (folder->files, lp->data);
-            g_object_ref (G_OBJECT (lp->data));
-          }
-
-      /* check if any files were added */
-      if (G_UNLIKELY (files != NULL))
+      if (value != NULL && !g_hash_table_contains (folder->files_map, thunar_file_get_file (THUNAR_FILE (value))))
         {
-          /* emit a "files-added" signal for the added files */
-          g_signal_emit (G_OBJECT (folder), folder_signals[FILES_ADDED], 0, files);
+          file = THUNAR_FILE (value);
 
-          /* release the added files list */
-          g_list_free (files);
+          /* put the file on the added list */
+          files = g_list_prepend (files, file);
+
+          g_hash_table_insert (folder->files_map, thunar_file_get_file (file), g_object_ref (file));
         }
+    }
 
-      /* determine all removed files (files on files, but not on new_files) */
-      for (files = NULL, lp = folder->files; lp != NULL; )
+  /* check if any files were added */
+  if (G_UNLIKELY (files != NULL))
+    {
+      /* emit a "files-added" signal for the added files */
+      g_signal_emit (G_OBJECT (folder), folder_signals[FILES_ADDED], 0, files);
+
+      /* start loading the content types of all files in the folder */
+      thunar_folder_load_content_types (folder, files);
+
+      /* release the added files list */
+      g_list_free (files);
+      files = NULL;
+    }
+
+  /* this is to handle removed files after a folder reload */
+  /* determine all removed files (files on files, but not on new_files) */
+  g_hash_table_iter_init (&iter, folder->files_map);
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      if (value != NULL)
         {
           /* determine the file */
-          file = THUNAR_FILE (lp->data);
-
-          /* determine the next list item */
-          lp = lp->next;
+          file = THUNAR_FILE (value);
 
           /* check if the file is not on new_files */
-          if (g_list_find (folder->new_files, file) == NULL)
-            {
-              /* put the file on the removed list (owns the reference now) */
-              files = g_list_prepend (files, file);
+          if (g_hash_table_contains (folder->new_files_map, thunar_file_get_file (file)))
+            continue;
 
-              /* remove from the internal files list */
-              folder->files = g_list_remove (folder->files, file);
-            }
+          /* put the file on the removed list (owns the reference now) */
+          files = g_list_prepend (files, g_object_ref (file));
         }
-
-      /* check if any files were removed */
-      if (G_UNLIKELY (files != NULL))
-        {
-          /* emit a "files-removed" signal for the removed files */
-          g_signal_emit (G_OBJECT (folder), folder_signals[FILES_REMOVED], 0, files);
-
-          /* release the removed files list */
-          thunar_g_list_free_full (files);
-        }
-
-      /* drop the temporary new_files list */
-      thunar_g_list_free_full (folder->new_files);
-      folder->new_files = NULL;
     }
-  else
+
+  for (lp = files; lp != NULL; lp = lp->next)
     {
-      /* just use the new files for the files list */
-      folder->files = folder->new_files;
-      folder->new_files = NULL;
+      file = THUNAR_FILE (lp->data);
 
-      if (folder->files != NULL)
-        {
-          /* emit a "files-added" signal for the new files */
-          g_signal_emit (G_OBJECT (folder), folder_signals[FILES_ADDED], 0, folder->files);
-        }
+      /* remove from the hashmap too */
+      g_hash_table_remove (folder->files_map, thunar_file_get_file (file));
     }
 
+  /* check if any files were removed */
+  if (G_UNLIKELY (files != NULL))
+    {
+      /* emit a "files-removed" signal for the removed files */
+      g_signal_emit (G_OBJECT (folder), folder_signals[FILES_REMOVED], 0, files);
+
+      /* release the removed files list */
+      thunar_g_list_free_full (files);
+    }
+
+  /* drop all mappings for new_files list too */
+  g_hash_table_remove_all (folder->new_files_map);
+
   /* schedule a reload of the file information of all files if requested */
   if (folder->reload_info)
     {
       folder->reload_info = FALSE;
-      for (lp = folder->files; lp != NULL; lp = lp->next)
-        thunar_file_reload (lp->data);
+      g_hash_table_iter_init (&iter, folder->files_map);
+      while (g_hash_table_iter_next (&iter, &key, &value))
+        {
+          if (value != NULL)
+            thunar_file_reload (THUNAR_FILE (value));
+        }
 
-      /* reload folder information too */
-      if (thunar_file_reload (folder->corresponding_file))
-        return;
+      /* block 'file-changed' signals of the folder itself until reload is done, in order to prevent recursion */
+      g_signal_handlers_block_by_func (G_OBJECT (folder->file_monitor), G_CALLBACK (thunar_folder_file_changed), folder);
 
+      /* reload folder itself */
+      thunar_file_reload (folder->corresponding_file);
+
+      /* listen to 'file-changed' signals of the folder itself again when reload is done */
+      g_signal_handlers_unblock_by_func (G_OBJECT (folder->file_monitor), G_CALLBACK (thunar_folder_file_changed), folder);
     }
 
   /* we did it, the folder is loaded */
@@ -602,9 +585,6 @@ thunar_folder_finished (ExoJob       *job,
       folder->job = NULL;
     }
 
-  /* restart the content type idle loader */
-  thunar_folder_content_type_loader (folder);
-
   /* tell the consumers that we have loaded the directory */
   g_object_notify (G_OBJECT (folder), "loading");
 }
@@ -635,9 +615,7 @@ thunar_folder_file_destroyed (ThunarFileMonitor *file_monitor,
                               ThunarFile        *file,
                               ThunarFolder      *folder)
 {
-  GList     files;
-  GList    *lp;
-  gboolean  restart = FALSE;
+  GList    files;
 
   _thunar_return_if_fail (THUNAR_IS_FILE (file));
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
@@ -650,29 +628,15 @@ thunar_folder_file_destroyed (ThunarFileMonitor *file_monitor,
       if (!folder->in_destruction)
         g_object_run_dispose (G_OBJECT (folder));
     }
-  else
+  else if (g_hash_table_contains (folder->files_map, thunar_file_get_file (file)))
     {
-      /* check if we have that file */
-      lp = g_list_find (folder->files, file);
-      if (G_LIKELY (lp != NULL))
-        {
-          if (folder->content_type_idle_id != 0)
-            restart = g_source_remove (folder->content_type_idle_id);
+      /* tell everybody that the file is gone */
+      files.data = file;
+      files.next = files.prev = NULL;
+      g_signal_emit (G_OBJECT (folder), folder_signals[FILES_REMOVED], 0, &files);
 
-          /* remove the file from our list */
-          folder->files = g_list_delete_link (folder->files, lp);
-
-          /* tell everybody that the file is gone */
-          files.data = file; files.next = files.prev = NULL;
-          g_signal_emit (G_OBJECT (folder), folder_signals[FILES_REMOVED], 0, &files);
-
-          /* drop our reference to the file */
-          g_object_unref (G_OBJECT (file));
-
-          /* continue collecting the metadata */
-          if (restart)
-            thunar_folder_content_type_loader (folder);
-        }
+      /* remove the file from our list */
+      g_hash_table_remove (folder->files_map, thunar_file_get_file (file));
     }
 }
 
@@ -683,13 +647,13 @@ static void
 thunar_file_infos_equal (ThunarFile *file,
                          GFile      *event_file)
 {
-  gchar     **attrs;
-  GFileInfo  *info1 = G_FILE_INFO (file->info);
-  GFileInfo  *info2;
-  guint       i;
-  gchar      *attr1, *attr2;
-  gboolean    printed = FALSE;
-  gchar      *bname;
+  gchar    **attrs;
+  GFileInfo *info1 = G_FILE_INFO (file->info);
+  GFileInfo *info2;
+  guint      i;
+  gchar     *attr1, *attr2;
+  gboolean   printed = FALSE;
+  gchar     *bname;
 
   attrs = g_file_info_list_attributes (info1, NULL);
   info2 = g_file_query_info (event_file, THUNARX_FILE_INFO_NAMESPACE,
@@ -743,11 +707,9 @@ thunar_folder_monitor (GFileMonitor     *monitor,
                        gpointer          user_data)
 {
   ThunarFolder *folder = THUNAR_FOLDER (user_data);
-  ThunarFile   *file;
-  ThunarFile   *other_parent;
-  GList        *lp;
+  ThunarFile   *file = NULL;
+  ThunarFile   *file_in_map;
   GList         list;
-  gboolean      restart = FALSE;
 
   _thunar_return_if_fail (G_IS_FILE_MONITOR (monitor));
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
@@ -755,110 +717,132 @@ thunar_folder_monitor (GFileMonitor     *monitor,
   _thunar_return_if_fail (THUNAR_IS_FILE (folder->corresponding_file));
   _thunar_return_if_fail (G_IS_FILE (event_file));
 
-  /* check on which file the event occurred */
-  if (!g_file_equal (event_file, thunar_file_get_file (folder->corresponding_file)))
-    {
-      /* check if we already ship the file */
-      for (lp = folder->files; lp != NULL; lp = lp->next)
-        if (g_file_equal (event_file, thunar_file_get_file (lp->data)))
-          break;
-
-      /* stop the content type collector */
-      if (folder->content_type_idle_id != 0)
-        restart = g_source_remove (folder->content_type_idle_id);
-
-      /* if we don't have it, add it if the event is not an "deleted" event */
-      if (G_UNLIKELY (lp == NULL && event_type != G_FILE_MONITOR_EVENT_DELETED))
-        {
-          /* allocate a file for the path */
-          file = thunar_file_get (event_file, NULL);
-          if (G_UNLIKELY (file != NULL))
-            {
-              /* prepend it to our internal list */
-              folder->files = g_list_prepend (folder->files, file);
-
-              /* tell others about the new file */
-              list.data = file; list.next = list.prev = NULL;
-              g_signal_emit (G_OBJECT (folder), folder_signals[FILES_ADDED], 0, &list);
-
-              /* load the new file */
-              thunar_file_reload (file);
-            }
-        }
-      else if (lp != NULL)
-        {
-          if (event_type == G_FILE_MONITOR_EVENT_DELETED)
-            {
-              ThunarFile *destroyed;
-
-              /* destroy the file */
-              thunar_file_destroy (lp->data);
-
-              /* if the file has not been destroyed by now, reload it to invalidate it */
-              destroyed = thunar_file_cache_lookup (event_file);
-              if (destroyed != NULL)
-                {
-                  thunar_file_reload (destroyed);
-                  g_object_unref (destroyed);
-                }
-            }
-          else if (event_type == G_FILE_MONITOR_EVENT_RENAMED ||
-                   event_type == G_FILE_MONITOR_EVENT_MOVED_IN ||
-                   event_type == G_FILE_MONITOR_EVENT_MOVED_OUT)
-            {
-              /* destroy the old file and update the new one */
-              thunar_file_destroy (lp->data);
-              if (other_file != NULL)
-                {
-                  file = thunar_file_get(other_file, NULL);
-                  if (file != NULL && THUNAR_IS_FILE (file))
-                    {
-                      if (thunar_file_reload (file))
-                        {
-                          /* if source and target folders are different, also tell
-                             the target folder to reload for the changes */
-                          if (thunar_file_has_parent (file))
-                            {
-                              other_parent = thunar_file_get_parent (file, NULL);
-                              if (other_parent &&
-                                  !g_file_equal (thunar_file_get_file(folder->corresponding_file),
-                                                 thunar_file_get_file(other_parent)))
-                                {
-                                  thunar_file_reload (other_parent);
-                                  g_object_unref (other_parent);
-                                }
-                            }
-                        }
-
-                      /* drop reference on the other file */
-                      g_object_unref (file);
-                    }
-                }
-            }
-          else
-            {
-#if DEBUG_FILE_CHANGES
-              thunar_file_infos_equal (lp->data, event_file);
-#endif
-              thunar_file_reload (lp->data);
-            }
-        }
-
-      /* check if we need to restart the collector */
-      if (restart)
-        thunar_folder_content_type_loader (folder);
-    }
-  else
+  if (g_file_equal (event_file, thunar_file_get_file (folder->corresponding_file)))
     {
       /* update/destroy the corresponding file */
       if (event_type == G_FILE_MONITOR_EVENT_DELETED)
+        thunar_file_destroy (folder->corresponding_file);
+      else
+        thunar_file_reload (folder->corresponding_file);
+      return;
+    }
+
+  file_in_map = g_hash_table_lookup (folder->files_map, event_file);
+
+  /* if we don't have it, add it if the event does not "delete" the "event_file" */
+  if (file_in_map == NULL && event_type != G_FILE_MONITOR_EVENT_DELETED && event_type != G_FILE_MONITOR_EVENT_MOVED_OUT)
+    {
+      if (event_type == G_FILE_MONITOR_EVENT_RENAMED)
         {
-          if (!thunar_file_exists (folder->corresponding_file))
-            thunar_file_destroy (folder->corresponding_file);
+          if (G_LIKELY (other_file != NULL))
+            {
+              ThunarFile *other_file_in_map;
+              other_file_in_map = g_hash_table_lookup (folder->files_map, other_file);
+
+              /* create a renamed file only if it doesn't exist */
+              if (!other_file_in_map)
+                file = thunar_file_get (other_file, NULL);
+            }
         }
       else
         {
-          thunar_file_reload (folder->corresponding_file);
+          file = thunar_file_get (event_file, NULL);
+        }
+
+      /* the file should not exist in file cache, so it's (re)loaded now */
+      if (file != NULL)
+        {
+          /* add a mapping of (gfile -> ThunarFile) to the hashmap */
+          g_hash_table_insert (folder->files_map, thunar_file_get_file (file), g_object_ref (file));
+
+          /* tell others about the new file */
+          list.data = file;
+          list.next = list.prev = NULL;
+          g_signal_emit (G_OBJECT (folder), folder_signals[FILES_ADDED], 0, &list);
+
+          if (other_file != NULL)
+            {
+              /* notify the thumbnail cache that we can now also move the thumbnail */
+              if (event_type == G_FILE_MONITOR_EVENT_MOVED_IN)
+                thunar_file_move_thumbnail_cache_file (other_file, event_file);
+              else
+                thunar_file_move_thumbnail_cache_file (event_file, other_file);
+            }
+        }
+    }
+  else if (file_in_map != NULL)
+    {
+      if (event_type == G_FILE_MONITOR_EVENT_DELETED)
+        {
+          /* destroy the file */
+          thunar_file_destroy (file_in_map);
+        }
+      else if (event_type == G_FILE_MONITOR_EVENT_RENAMED || event_type == G_FILE_MONITOR_EVENT_MOVED_IN || event_type == G_FILE_MONITOR_EVENT_MOVED_OUT)
+        {
+          if (event_type == G_FILE_MONITOR_EVENT_MOVED_IN)
+            {
+              /* reload existing file, the case when file doesn't exist is handled above */
+              thunar_file_reload (file_in_map);
+            }
+          else if (event_type == G_FILE_MONITOR_EVENT_MOVED_OUT)
+            {
+              /* destroy the old file */
+              thunar_file_destroy (file_in_map);
+            }
+          else if (event_type == G_FILE_MONITOR_EVENT_RENAMED && G_LIKELY (other_file != NULL))
+            {
+              ThunarFile *dest_file_in_map;
+
+              /* check if we already ship the destination file */
+              dest_file_in_map = g_hash_table_lookup (folder->files_map, other_file);
+
+              if (dest_file_in_map)
+                {
+                  /* destroy source file if the destination file already exists
+                      to prevent duplicated file */
+                  if (file_in_map == dest_file_in_map)
+                    {
+                      g_warning ("Same g_file for source and destination file during rename");
+                    }
+                  else
+                    {
+                      thunar_file_destroy (file_in_map);
+                      file = dest_file_in_map;
+                    }
+                }
+              else
+                {
+                  file = file_in_map;
+
+                  /* remove the old reference from the hash table before it becomes invalid;
+                   * during thunar_file_replace_file call */
+                  g_hash_table_remove (folder->files_map, event_file);
+
+                  /* replace GFile in ThunarFile for the renamed file */
+                  thunar_file_replace_file (file, other_file);
+
+                  /* insert new mapping of (gfile, ThunarFile) for the newly renamed file */
+                  g_hash_table_insert (folder->files_map, other_file, g_object_ref (file));
+
+
+                }
+
+              /* reload the renamed file */
+              if (thunar_file_reload (file))
+                {
+                  /* notify the thumbnail cache that we can now also move the thumbnail */
+                  thunar_file_move_thumbnail_cache_file (event_file, other_file);
+                }
+            }
+        }
+      /* This should handle G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT &
+       * G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED */
+      else
+        {
+#if DEBUG_FILE_CHANGES
+          thunar_file_infos_equal (file_in_map, event_file);
+#endif
+          thunar_file_reload (file_in_map);
         }
     }
 }
@@ -879,7 +863,7 @@ thunar_folder_monitor (GFileMonitor     *monitor,
  * Return value: the #ThunarFolder which corresponds
  *               to @file.
  **/
-ThunarFolder*
+ThunarFolder *
 thunar_folder_get_for_file (ThunarFile *file)
 {
   ThunarFolder *folder;
@@ -935,7 +919,7 @@ thunar_folder_get_for_file (ThunarFile *file)
  *
  * Return value: the #ThunarFile corresponding to @folder.
  **/
-ThunarFile*
+ThunarFile *
 thunar_folder_get_corresponding_file (const ThunarFolder *folder)
 {
   _thunar_return_val_if_fail (THUNAR_IS_FOLDER (folder), NULL);
@@ -949,15 +933,16 @@ thunar_folder_get_corresponding_file (const ThunarFolder *folder)
  * @folder : a #ThunarFolder instance.
  *
  * Returns the list of files currently known for @folder.
- * The returned list is owned by @folder and may not be freed!
+ * The content of the list is owned by the hash table and should not be modified or freed. Use g_list_free() when done using the list.
+ * The caller of the function takes ownership of the data container, but not the data inside it.
  *
- * Return value: the list of #ThunarFiles for @folder.
+ * Returns: (transfer container): A GList containing all the #ThunarFiles inside the hash table. Use g_list_free() when done using the list.
  **/
-GList*
+GList *
 thunar_folder_get_files (const ThunarFolder *folder)
 {
   _thunar_return_val_if_fail (THUNAR_IS_FOLDER (folder), NULL);
-  return folder->files;
+  return g_hash_table_get_values (folder->files_map);
 }
 
 
@@ -1014,9 +999,13 @@ thunar_folder_reload (ThunarFolder *folder,
   /* reload file info too? */
   folder->reload_info = reload_info;
 
-  /* stop metadata collector */
-  if (folder->content_type_idle_id != 0)
-    g_source_remove (folder->content_type_idle_id);
+  /* stop content type loading */
+  if (G_UNLIKELY (folder->content_type_job != NULL))
+    {
+      exo_job_cancel (EXO_JOB (folder->content_type_job));
+      g_object_unref (folder->content_type_job);
+      folder->content_type_job = NULL;
+    }
 
   /* check if we are currently connect to a job */
   if (G_UNLIKELY (folder->job != NULL))
@@ -1027,9 +1016,8 @@ thunar_folder_reload (ThunarFolder *folder,
       folder->job = NULL;
     }
 
-  /* reset the new_files list */
-  thunar_g_list_free_full (folder->new_files);
-  folder->new_files = NULL;
+  /* reset the new_files list & new_files_map hash table */
+  g_hash_table_remove_all (folder->new_files_map);
 
   /* start a new job */
   folder->job = thunar_io_jobs_list_directory (thunar_file_get_file (folder->corresponding_file));
