@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <gio/gio.h>
@@ -39,11 +39,12 @@
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunar/thunar-file.h>
-#include <thunar/thunar-gio-extensions.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-util.h>
+#include "thunar/thunar-file.h"
+#include "thunar/thunar-gio-extensions.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-util.h"
+#include "thunar/thunar-file.h"
 
 
 
@@ -65,7 +66,7 @@ device_icon_name [] =
   { "drive-removable-media-usb"          , N_("USB Drive") },
 
   /* Freedesktop icon-naming-spec */
-  { "camera*"                , N_("Camera") },
+  { "camera-photo*"          , N_("Camera") },
   { "drive-harddisk*"        , N_("Harddisk") },
   { "drive-optical*"         , N_("Optical Drive") },
   { "drive-removable-media*" , N_("Removable Drive") },
@@ -81,8 +82,9 @@ device_icon_name [] =
 
 
 
-static const gchar *guess_device_type_from_icon_name (const gchar * icon_name);
-
+static const gchar     *guess_device_type_from_icon_name           (const gchar *icon_name);
+static       GFileInfo *thunar_g_file_get_content_type_querry_info (GFile       *gfile,
+                                                                    GError      *err);
 
 
 GFile *
@@ -1463,4 +1465,292 @@ thunar_g_file_get_resolved_path (GFile *file)
 
   g_free (path);
   return real_path;
+}
+
+
+
+gboolean
+thunar_g_file_is_empty (GFile *file)
+{
+  GFileEnumerator *enumerator;
+  GError          *err = NULL;
+  gboolean         is_empty = TRUE;
+  GFileInfo       *info;
+
+  _thunar_return_val_if_fail (G_IS_FILE (file), TRUE);
+
+  enumerator = g_file_enumerate_children (file, NULL,
+                                          G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                          NULL, &err);
+
+  if (err != NULL)
+    {
+      g_warning ("Failed to check if the file has any children!");
+      return TRUE;
+    }
+
+  if (g_file_enumerator_iterate (enumerator, &info, NULL, NULL, &err) && info != NULL)
+    is_empty = FALSE;
+
+  g_object_unref (enumerator);
+
+  return is_empty;
+}
+
+
+
+static GFileInfo*
+thunar_g_file_get_content_type_querry_info (GFile *gfile,
+                                            GError *err)
+{
+  GFileInfo   *info = NULL;
+
+  info = g_file_query_info (gfile,
+                            G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+                            G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
+                            G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+                            G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
+                            G_FILE_QUERY_INFO_NONE,
+                            NULL, &err);
+  return info;
+}
+
+
+
+/**
+ * thunar_g_file_get_content_type:
+ * @file : #GFile for which the content type will be returned
+ *
+ * Gets the content type of the passed #GFile. Will always return a valid string.
+ *
+ * Return value: (transfer full): The content type as #gchar 
+ * The returned string should be freed with g_free() when no longer needed.
+ **/
+char*
+thunar_g_file_get_content_type (GFile *gfile)
+{
+  gboolean     is_symlink = FALSE;
+  GFileInfo   *info = NULL;
+  GError      *err = NULL;
+  gchar       *content_type = NULL;
+
+  info = thunar_g_file_get_content_type_querry_info (gfile, err);
+
+  /* follow symlink, if found */
+  if (G_LIKELY (info != NULL))
+    {
+      is_symlink = g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK);
+      if (is_symlink)
+        {
+          GFile *link_target = thunar_g_file_new_for_symlink_target (gfile);
+          g_object_unref (info);
+          info = NULL;
+          if (link_target != NULL)
+            info = thunar_g_file_get_content_type_querry_info (link_target, err);
+          g_object_unref (link_target);
+        }
+    }
+
+  if (G_LIKELY (info != NULL))
+    {
+      if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
+        {
+          /* this we known for sure */
+          content_type = g_strdup ("inode/directory");
+        }
+      else
+        {
+          content_type = g_strdup (g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE));
+          if (G_UNLIKELY (content_type == NULL))
+            content_type = g_strdup (g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE));
+        }
+
+      g_object_unref (info);
+    }
+  else
+    {
+      /* If gfile retrieved above is NULL, then g_file_query_info won't be called, thus keeping info NULL.
+        * In this case, err will also be NULL. So it will fallback to "unknown" mime-type */
+      if (G_LIKELY (err != NULL))
+        {
+          /* The mime-type 'inode/symlink' is  only used for broken links.
+            * When the link is functional, the mime-type of the link target will be used */
+          if (G_LIKELY (is_symlink && err->code == G_IO_ERROR_NOT_FOUND))
+              content_type = g_strdup ("inode/symlink");
+          else
+              g_warning ("Content type loading failed for %s: %s",
+                          g_file_get_uri (gfile),
+                          err->message);
+          g_error_free (err);
+        }
+    }
+
+  /* fallback */
+  if (content_type == NULL)
+    content_type = g_strdup (DEFAULT_CONTENT_TYPE);
+
+  return content_type;
+}
+
+
+
+static void
+thunar_g_file_set_metadata_setting_finish (GObject      *source_object,
+                                           GAsyncResult *result,
+                                           gpointer      user_data)
+{
+  GFile      *file = NULL;
+  GError     *error = NULL;
+  ThunarFile *thunar_file = NULL;
+
+  file = G_FILE (source_object);
+
+  if (file == NULL)
+    return;
+
+  if (!g_file_set_attributes_finish (file, result, NULL, &error))
+    {
+      g_warning ("Failed to set metadata: %s", error->message);
+      g_error_free (error);
+    }
+
+  thunar_file = thunar_file_get (file, NULL);
+
+  if (thunar_file == NULL)
+    return;
+
+  thunar_file_changed (thunar_file);
+  g_object_unref (thunar_file);
+}
+
+
+
+/**
+ * thunar_g_file_set_metadata_setting:
+ * @file          : a #GFile instance.
+ * @setting_name  : the name of the setting to set
+ * @setting_value : the value to set
+ * @async         : whether g_file_set_attributes_async or g_file_set_attributes_from_info should be used
+ *
+ * Sets the setting @setting_name of @file to @setting_value and stores it in
+ * the @file<!---->s metadata.
+ **/
+void
+thunar_g_file_set_metadata_setting (GFile       *file,
+                                    GFileInfo   *info,
+                                    const gchar *setting_name,
+                                    const gchar *setting_value,
+                                    gboolean     async)
+{
+  GFileInfo *info_new;
+  gchar     *attr_name;
+
+  _thunar_return_if_fail (G_IS_FILE (file));
+  _thunar_return_if_fail (G_IS_FILE_INFO (info));
+
+  /* convert the setting name to an attribute name */
+  attr_name = g_strdup_printf ("metadata::thunar-%s", setting_name);
+
+  /* set the value in the current info. this call is needed to update the in-memory
+   * GFileInfo structure to ensure that the new attribute value is available immediately */
+  g_file_info_set_attribute_string (info, attr_name, setting_value);
+
+  /* send meta data to the daemon. this call is needed to store the new value of
+   * the attribute in the file system */
+  info_new = g_file_info_new ();
+  g_file_info_set_attribute_string (info_new, attr_name, setting_value);
+  if (async)
+    {
+      g_file_set_attributes_async (file, info_new,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   G_PRIORITY_DEFAULT,
+                                   NULL,
+                                   thunar_g_file_set_metadata_setting_finish,
+                                   NULL);
+    }
+  else
+    {
+      g_file_set_attributes_from_info (file, info_new,
+                                       G_FILE_QUERY_INFO_NONE,
+                                       NULL,
+                                       NULL);
+    }
+  g_free (attr_name);
+  g_object_unref (G_OBJECT (info_new));
+}
+
+
+
+/**
+ * thunar_file_clear_metadata_setting:
+ * @file          : a #GFile instance.
+ * @setting_name  : the name of the setting to clear
+ *
+ * Clear the metadata setting @setting_name of @file
+ **/
+void
+thunar_g_file_clear_metadata_setting (GFile       *file,
+                                      GFileInfo   *info,
+                                      const gchar *setting_name)
+{
+  gchar *attr_name;
+
+  _thunar_return_if_fail (G_IS_FILE (file));
+
+  if (info == NULL)
+    return;
+
+  /* convert the setting name to an attribute name */
+  attr_name = g_strdup_printf ("metadata::thunar-%s", setting_name);
+
+  if (!g_file_info_has_attribute (info, attr_name))
+    {
+      g_free (attr_name);
+      return;
+    }
+
+  g_file_info_remove_attribute (info, attr_name);
+  g_file_set_attribute (file, attr_name, G_FILE_ATTRIBUTE_TYPE_INVALID,
+                        NULL, G_FILE_QUERY_INFO_NONE, NULL, NULL);
+  g_free (attr_name);
+}
+
+
+
+/**
+ * thunar_file_get_metadata_setting:
+ * @file         : a #GFile instance.
+ * @setting_name : the name of the setting to get
+ *
+ * Gets the stored value of the metadata setting @setting_name for @file. Returns %NULL
+ * if there is no stored setting.
+ *
+ * Return value: (transfer none): the stored value of the setting for @file, or %NULL
+ **/
+const gchar *
+thunar_g_file_get_metadata_setting (GFile       *file,
+                                    GFileInfo   *info,
+                                    const gchar *setting_name)
+{
+  gchar       *attr_name;
+  const gchar *attr_value;
+
+  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
+
+  if (info == NULL)
+    return NULL;
+
+  /* convert the setting name to an attribute name */
+  attr_name = g_strdup_printf ("metadata::thunar-%s", setting_name);
+
+  if (!g_file_info_has_attribute (info, attr_name))
+    {
+      g_free (attr_name);
+      return NULL;
+    }
+
+  attr_value = g_file_info_get_attribute_string (info, attr_name);
+  g_free (attr_name);
+
+  return attr_value;
 }
\ No newline at end of file
