@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_ERRNO_H
@@ -29,18 +29,20 @@
 #include <gio/gio.h>
 #include <glib/gstdio.h>
 
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-enum-types.h>
-#include <thunar/thunar-file.h>
-#include <thunar/thunar-gio-extensions.h>
-#include <thunar/thunar-io-jobs-util.h>
-#include <thunar/thunar-io-jobs.h>
-#include <thunar/thunar-io-scan-directory.h>
-#include <thunar/thunar-job.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-simple-job.h>
-#include <thunar/thunar-thumbnail-cache.h>
-#include <thunar/thunar-transfer-job.h>
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-enum-types.h"
+#include "thunar/thunar-file.h"
+#include "thunar/thunar-gio-extensions.h"
+#include "thunar/thunar-io-jobs-util.h"
+#include "thunar/thunar-io-jobs.h"
+#include "thunar/thunar-io-scan-directory.h"
+#include "thunar/thunar-job.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-simple-job.h"
+#include "thunar/thunar-thumbnail-cache.h"
+#include "thunar/thunar-transfer-job.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-gobject-extensions.h"
 
 
 
@@ -1500,3 +1502,387 @@ thunar_io_jobs_count_files (ThunarFile *file)
   return thunar_simple_job_new (_thunar_io_jobs_count, 1,
                                 THUNAR_TYPE_FILE, file);
 }
+
+
+
+static void
+_thunar_search_folder (ThunarStandardViewModel           *model,
+                       ThunarJob                         *job,
+                       gchar                             *uri,
+                       gchar                            **search_query_c_terms,
+                       enum ThunarStandardViewModelSearch search_type,
+                       gboolean                           show_hidden)
+{
+  GCancellable    *cancellable;
+  GFileEnumerator *enumerator;
+  GFile           *directory;
+  GList           *files_found = NULL; /* contains the matching files in this folder only */
+  const gchar *namespace;
+  const gchar *display_name;
+  gchar       *display_name_c; /* converted to ignore case */
+
+  cancellable = exo_job_get_cancellable (EXO_JOB (job));
+  directory = g_file_new_for_uri (uri);
+  namespace = G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+              G_FILE_ATTRIBUTE_STANDARD_TARGET_URI ","
+              G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
+              G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP ","
+              G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN ","
+              G_FILE_ATTRIBUTE_STANDARD_NAME ", recent::*";
+
+  /* The directory enumerator MUST NOT follow symlinks itself, meaning that any symlinks that
+   * g_file_enumerator_next_file() emits are the actual symlink entries. This prevents one
+   * possible source of infinitely deep recursion.
+   *
+   * There is otherwise no special handling of entries in the folder which are symlinks,
+   * which allows them to appear in the search results. */
+  enumerator = g_file_enumerate_children (directory, namespace, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, cancellable, NULL);
+  if (enumerator == NULL)
+    return;
+
+  /* go through every file in the folder and check if it matches */
+  while (exo_job_is_cancelled (EXO_JOB (job)) == FALSE)
+    {
+      GFile     *file;
+      GFileInfo *info;
+      GFileType  type;
+
+      /* get GFile and GFileInfo */
+      info = g_file_enumerator_next_file (enumerator, cancellable, NULL);
+      if (G_UNLIKELY (info == NULL))
+        break;
+
+      if (g_file_has_uri_scheme (directory, "recent"))
+        {
+          file = g_file_new_for_uri (g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_TARGET_URI));
+          g_object_unref (info);
+          info = g_file_query_info (file, namespace, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, cancellable, NULL);
+          if (G_UNLIKELY (info == NULL))
+            break;
+        }
+      else
+        file = g_file_get_child (directory, g_file_info_get_name (info));
+
+      /* respect last-show-hidden */
+      if (show_hidden == FALSE)
+        {
+          /* same logic as thunar_file_is_hidden() */
+          if (g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN)
+              || g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP))
+            {
+              g_object_unref (file);
+              g_object_unref (info);
+              continue;
+            }
+        }
+
+      type = g_file_info_get_file_type (info);
+
+      /* handle directories */
+      if (type == G_FILE_TYPE_DIRECTORY && search_type == THUNAR_STANDARD_VIEW_MODEL_SEARCH_RECURSIVE)
+        {
+          _thunar_search_folder (model, job, g_file_get_uri (file), search_query_c_terms, search_type, show_hidden);
+        }
+
+      /* prepare entry display name */
+      display_name = g_file_info_get_display_name (info);
+      display_name_c = thunar_g_utf8_normalize_for_search (display_name, TRUE, TRUE);
+
+      /* search for all substrings */
+      if (thunar_util_search_terms_match (search_query_c_terms, display_name_c))
+        files_found = g_list_prepend (files_found, thunar_file_get (file, NULL));
+
+      /* free memory */
+      g_free (display_name_c);
+      g_object_unref (file);
+      g_object_unref (info);
+    }
+
+  g_object_unref (enumerator);
+  g_object_unref (directory);
+
+  if (exo_job_is_cancelled (EXO_JOB (job)))
+    return;
+
+  thunar_standard_view_model_add_search_files (model, files_found);
+}
+
+
+
+static gboolean
+_thunar_job_search_directory (ThunarJob *job,
+                              GArray    *param_values,
+                              GError   **error)
+{
+  ThunarStandardViewModel           *model;
+  ThunarFile                        *directory;
+  const char                        *search_query_c;
+  gchar                            **search_query_c_terms;
+  gboolean                           is_source_device_local;
+  ThunarRecursiveSearchMode          mode;
+  gboolean                           show_hidden;
+  enum ThunarStandardViewModelSearch search_type;
+  gchar                             *directory_uri;
+
+  search_type = THUNAR_STANDARD_VIEW_MODEL_SEARCH_NON_RECURSIVE;
+
+  if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
+    return FALSE;
+
+  model = g_value_get_object (&g_array_index (param_values, GValue, 0));
+  search_query_c = g_value_get_string (&g_array_index (param_values, GValue, 1));
+  directory = g_value_get_object (&g_array_index (param_values, GValue, 2));
+  mode = g_value_get_enum (&g_array_index (param_values, GValue, 3));
+  show_hidden = g_value_get_boolean (&g_array_index (param_values, GValue, 4));
+
+  search_query_c_terms = thunar_util_split_search_query (search_query_c, error);
+  if (search_query_c_terms == NULL)
+    return FALSE;
+
+  is_source_device_local = thunar_g_file_is_on_local_device (thunar_file_get_file (directory));
+  if (mode == THUNAR_RECURSIVE_SEARCH_ALWAYS || (mode == THUNAR_RECURSIVE_SEARCH_LOCAL && is_source_device_local))
+    search_type = THUNAR_STANDARD_VIEW_MODEL_SEARCH_RECURSIVE;
+
+  directory_uri = thunar_file_dup_uri (directory);
+
+  _thunar_search_folder (model, job, directory_uri, search_query_c_terms, search_type, show_hidden);
+
+  g_free (directory_uri);
+  g_strfreev (search_query_c_terms);
+
+  return TRUE;
+}
+
+
+
+ThunarJob *
+thunar_io_jobs_search_directory (ThunarStandardViewModel *model,
+                                 const gchar             *search_query,
+                                 ThunarFile              *directory)
+{
+  ThunarPreferences        *preferences;
+  ThunarRecursiveSearchMode mode;
+  gboolean                  show_hidden;
+
+  preferences = thunar_preferences_get ();
+
+  /* grab a reference of preferences determine the current recursive search mode */
+  g_object_get (G_OBJECT (preferences), "misc-recursive-search", &mode, NULL);
+  g_object_get (G_OBJECT (preferences), "last-show-hidden", &show_hidden, NULL);
+
+  g_object_unref (preferences);
+  return thunar_simple_job_new (_thunar_job_search_directory, 5,
+                                THUNAR_TYPE_STANDARD_VIEW_MODEL, model,
+                                G_TYPE_STRING, search_query,
+                                THUNAR_TYPE_FILE, directory,
+                                G_TYPE_ENUM, mode,
+                                G_TYPE_BOOLEAN, show_hidden);
+}
+
+
+
+static gboolean
+_thunar_io_jobs_clear_metadata_for_files (ThunarJob *job,
+                                          GArray    *param_values,
+                                          GError   **error)
+{
+  GList *gfiles, *infos;
+  GList *setting_names;
+
+  gfiles = g_value_get_pointer (&g_array_index (param_values, GValue, 0));
+  infos = g_value_get_pointer (&g_array_index (param_values, GValue, 1));
+  setting_names = g_value_get_pointer (&g_array_index (param_values, GValue, 2));
+
+  for (GList *gfile = gfiles, *info = infos; gfile != NULL && info != NULL; gfile = gfile->next, info = info->next)
+    for (GList *sn = setting_names; sn != NULL; sn = sn->next)
+      thunar_g_file_clear_metadata_setting (gfile->data, info->data, sn->data);
+
+  g_list_free_full (gfiles, g_object_unref);
+  g_list_free_full (infos, g_object_unref);
+  g_list_free_full (setting_names, g_free);
+
+  return TRUE;
+}
+
+
+
+/**
+ * thunar_io_jobs_clear_metadata_for_files:
+ * @files: a #GList of #ThunarFiles
+ * @first_metadata_setting_name: the setting name to set
+ * @...: followed by the corresponding setting value and more
+ *       setting_name/value pairs, finally ending with a %NULL
+ *
+ * Accepts a variable length metadata_setting_names to clear
+ * for @files.
+ *
+ * Note: the reason for making this func variable length is that
+ * by clearing multiple metadata at once we don't have to call
+ * this func for each metadata setting. Individual calls would trigger
+ * multiple threads which would then require a mutex logic to handle
+ * callback.
+ **/
+ThunarJob *
+thunar_io_jobs_clear_metadata_for_files (GList *files,
+                                         ...)
+{
+  va_list    list;
+  GList     *setting_names = NULL;
+  gchar     *str;
+  GList     *gfiles = NULL, *infos = NULL;
+
+  va_start (list, files);
+  str = va_arg (list, gchar *);
+  while (str != NULL)
+    {
+      setting_names = g_list_prepend (setting_names, g_strdup (str));
+      str = va_arg (list, gchar *);
+    }
+  va_end (list);
+
+  for (GList *lp = files; lp != NULL; lp = lp->next)
+    {
+      gfiles = g_list_prepend (gfiles, g_object_ref (thunar_file_get_file (THUNAR_FILE (lp->data))));
+      infos = g_list_prepend (infos, g_object_ref (thunar_file_get_info (THUNAR_FILE (lp->data))));
+    }
+  return thunar_simple_job_new (_thunar_io_jobs_clear_metadata_for_files, 3,
+                                G_TYPE_POINTER, gfiles,
+                                G_TYPE_POINTER, infos,
+                                G_TYPE_POINTER, setting_names);
+}
+
+
+
+static gboolean
+_thunar_io_jobs_set_metadata_for_files (ThunarJob *job,
+                                        GArray    *param_values,
+                                        GError   **error)
+{
+  GList *gfiles, *infos;
+  GList *setting_value_pairs;
+
+  gfiles = g_value_get_pointer (&g_array_index (param_values, GValue, 0));
+  infos = g_value_get_pointer (&g_array_index (param_values, GValue, 1));
+  setting_value_pairs = g_value_get_pointer (&g_array_index (param_values, GValue, 2));
+
+  for (GList *gfile = gfiles, *info = infos; gfile != NULL && info != NULL; gfile = gfile->next, info = info->next)
+    for (GList *sn = setting_value_pairs; sn != NULL; sn = sn->next)
+      thunar_g_file_set_metadata_setting (gfile->data, info->data,
+                                          ((gchar **) sn->data)[0],
+                                          ((gchar **) sn->data)[1], FALSE);
+
+  g_list_free_full (gfiles, g_object_unref);
+  g_list_free_full (infos, g_object_unref);
+  g_list_free_full (setting_value_pairs, (GDestroyNotify) g_strfreev);
+
+  return TRUE;
+}
+
+
+
+/**
+ * thunar_io_jobs_set_metadata_for_files:
+ * @files: a #GList of #ThunarFiles
+ * @first_metadata_setting_name: the setting name to set
+ * @...: followed by the corresponding setting value and more
+ *       setting_name/value pairs, finally ending with a %NULL
+ *
+ * Accepts a variable length metadata setting_name
+ * & setting_value pairs to set for @files.
+ *
+ * Note: the reason for making this func variable length is that
+ * by clearing multiple metadata at once we don't have to call
+ * this func for each metadata setting. Individual calls would trigger
+ * multiple threads which would then require a mutex logic to handle
+ * callback.
+ **/
+ThunarJob *
+thunar_io_jobs_set_metadata_for_files (GList *files,
+                                       ...)
+{
+  va_list    list;
+  GList     *setting_value_pairs = NULL;
+  gchar     *str, *val;
+  gchar    **sn_val_pair;
+  GList     *gfiles = NULL, *infos = NULL;
+
+  /* this func accepts a variable length setting_name, setting_value pair*/
+  va_start (list, files);
+  str = va_arg (list, gchar *);
+  while (str != NULL)
+    {
+      val = va_arg (list, gchar *);
+      if (val == NULL)
+        break;
+      // Needs to be null terminated for use in g_strfreev
+      sn_val_pair = (gchar **) g_malloc0 (sizeof (gchar *) * 3);
+      sn_val_pair[0] = g_strdup (str);
+      sn_val_pair[1] = g_strdup (val);
+      sn_val_pair[2] = NULL;
+      setting_value_pairs = g_list_prepend (setting_value_pairs, sn_val_pair);
+      str = va_arg (list, gchar *);
+    }
+  va_end (list);
+
+  for (GList *lp = files; lp != NULL; lp = lp->next)
+    {
+      gfiles = g_list_prepend (gfiles, g_object_ref (thunar_file_get_file (THUNAR_FILE (lp->data))));
+      infos = g_list_prepend (infos, g_object_ref (thunar_file_get_info (THUNAR_FILE (lp->data))));
+    }
+
+  return thunar_simple_job_new (_thunar_io_jobs_set_metadata_for_files, 3,
+                                G_TYPE_POINTER, gfiles,
+                                G_TYPE_POINTER, infos,
+                                G_TYPE_POINTER, setting_value_pairs);
+}
+
+
+
+static gboolean
+_thunar_job_load_content_types (ThunarJob *job,
+                                GArray    *param_values,
+                                GError   **error)
+{
+  GList *thunar_files;
+
+  if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
+    return FALSE;
+
+  thunar_files = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
+
+  for (GList *lp = thunar_files; lp != NULL; lp = lp->next)
+    {
+      gchar *content_type;
+      GFile       *g_file;
+
+      g_file = thunar_file_get_file (THUNAR_FILE (lp->data));
+      content_type = thunar_g_file_get_content_type (g_file);
+      thunar_file_set_content_type (THUNAR_FILE (lp->data), content_type);
+      g_free (content_type);
+    }
+
+  return TRUE;
+}
+
+
+
+/**
+ * thunar_io_jobs_load_content_types:
+ * @files: a #GList of #ThunarFiles
+ *
+ * Loads the content types of the passed #ThunarFiles in a separate thread.
+ * When loaded,'thunar_file_set_content_type' is called for each #ThunarFile.
+ * Sice that happens from within a separate thread, 'thunar_file_set_content_type' has to care for mutex protection.
+ *
+ * Returns: (transfer none): the #ThunarJob which manages the separate thread
+ **/
+ThunarJob *
+thunar_io_jobs_load_content_types (GList *files)
+{
+  GList     *copy = thunar_g_list_copy_deep (files);
+  ThunarJob *job = thunar_simple_job_new (_thunar_job_load_content_types, 1,
+                                          THUNAR_TYPE_G_FILE_LIST, files);
+
+  g_signal_connect_swapped (job, "finished", G_CALLBACK (thunar_g_list_free_full), copy);
+  return job;
+}
