@@ -21,8 +21,10 @@
 #ifndef __THUNAR_IO_JOBS_H__
 #define __THUNAR_IO_JOBS_H__
 
-#include <thunar/thunar-job.h>
-#include <thunar/thunar-enum-types.h>
+#include "thunar/thunar-job.h"
+#include "thunar/thunar-enum-types.h"
+#include "thunar/thunar-standard-view-model.h"
+#include "thunar/thunar-job.h"
 
 G_BEGIN_DECLS
 
@@ -53,6 +55,14 @@ ThunarJob *thunar_io_jobs_rename_file      (ThunarFile            *file,
                                             const gchar           *display_name,
                                             ThunarOperationLogMode log_mode) G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
 ThunarJob *thunar_io_jobs_count_files      (ThunarFile            *file);
+ThunarJob *thunar_io_jobs_search_directory (ThunarStandardViewModel *model,
+                                            const gchar             *search_query,
+                                            ThunarFile              *directory);
+ThunarJob *thunar_io_jobs_clear_metadata_for_files (GList          *files,
+                                                    ...);
+ThunarJob *thunar_io_jobs_set_metadata_for_files   (GList          *files,
+                                                    ...);
+ThunarJob *thunar_io_jobs_load_content_types(GList                  *files) G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
 
 G_END_DECLS
 
