@@ -16,10 +16,10 @@
  * this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-io-jobs.h>
-#include <thunar/thunar-job-operation.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-io-jobs.h"
+#include "thunar/thunar-job-operation.h"
+#include "thunar/thunar-private.h"
 
 /**
  * SECTION:thunar-job-operation
@@ -436,7 +436,7 @@ thunar_job_operation_execute (ThunarJobOperation *job_operation,
           }
         else
           {
-            operation_canceled = thunar_application_unlink_files (application, NULL, thunar_file_list, TRUE, TRUE);
+            operation_canceled = thunar_application_unlink_files (application, NULL, thunar_file_list, TRUE, TRUE, THUNAR_OPERATION_LOG_OPERATIONS);
             g_list_free_full (thunar_file_list, g_object_unref);
           }
 
@@ -687,14 +687,14 @@ thunar_job_operation_restore_from_trash (ThunarJobOperation  *operation,
     /* (All files located in trash have symlinks resolved) */
     if (parent != NULL)
       {
-        GFile *parent_resolved = NULL;
+        gchar *real_path_parent = NULL;
         gchar *basename = g_file_get_basename (lp->data);
-        parent_resolved = thunar_g_file_resolve_symlink (parent);
+        real_path_parent = thunar_g_file_get_resolved_path (parent);
         g_object_unref (parent);
-        if (parent_resolved != NULL && basename != NULL)
-          real_path = g_build_filename (g_file_get_path (parent_resolved), basename, NULL);
+        if (real_path_parent != NULL && basename != NULL)
+          real_path = g_build_filename (real_path_parent, basename, NULL);
         g_free (basename);
-        g_object_unref (parent_resolved);
+        g_free (real_path_parent);
       }
 
     if (real_path != NULL)
@@ -732,7 +732,7 @@ thunar_job_operation_restore_from_trash (ThunarJobOperation  *operation,
       lookup = g_hash_table_lookup (files_trashed, original_file);
 
       /* if we deleted the file in this session, and the current file we're looking at was deleted
-       * during the time the operation occured, we conclude we found the right file */
+       * during the time the operation occurred, we conclude we found the right file */
       if (lookup != NULL && operation->start_timestamp <= deletion_time && deletion_time <= operation->end_timestamp)
         {
           trashed_file = g_file_get_child (trash, g_file_info_get_name (info));
@@ -761,3 +761,20 @@ thunar_job_operation_restore_from_trash (ThunarJobOperation  *operation,
 
   g_hash_table_unref (files_trashed);
 }
+
+
+
+/* thunar_job_operation_get_action_text:
+ * @operation: Instance of #ThunarJobOperation
+ *
+ * Returns a text describing the current operation 
+ * The caller is responsible to free the returned string using g_free() when no longer needed.
+ *
+ * Return value: pointer to the text string
+ **/
+gchar*
+thunar_job_operation_get_action_text (ThunarJobOperation *job_operation)
+{
+  guint  files_count = job_operation->source_file_list == NULL ? 0 : g_list_length (job_operation->source_file_list);
+  return g_strdup_printf (ngettext ("%d file", "%d files", files_count), files_count);
+}
