@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_MEMORY_H
@@ -31,14 +31,14 @@
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-dnd.h>
-#include <thunar/thunar-gio-extensions.h>
-#include <thunar/thunar-icon-factory.h>
-#include <thunar/thunar-location-button.h>
-#include <thunar/thunar-pango-extensions.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-dnd.h"
+#include "thunar/thunar-gio-extensions.h"
+#include "thunar/thunar-icon-factory.h"
+#include "thunar/thunar-location-button.h"
+#include "thunar/thunar-pango-extensions.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
 
 
 
