@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_MEMORY_H
@@ -35,16 +35,16 @@
 
 #include <exo/exo.h>
 
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-enum-types.h>
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-io-jobs.h>
-#include <thunar/thunar-pango-extensions.h>
-#include <thunar/thunar-permissions-chooser.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-user.h>
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-enum-types.h"
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-io-jobs.h"
+#include "thunar/thunar-pango-extensions.h"
+#include "thunar/thunar-permissions-chooser.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-user.h"
 
 
 
