@@ -18,30 +18,30 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4kbd-private/xfce-shortcuts-editor.h>
 #include <libxfce4kbd-private/xfce-shortcuts-editor-dialog.h>
 
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-compact-view.h>
-#include <thunar/thunar-details-view.h>
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-enum-types.h>
-#include <thunar/thunar-gdk-extensions.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-icon-view.h>
-#include <thunar/thunar-pango-extensions.h>
-#include <thunar/thunar-preferences-dialog.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-util.h>
-#include <thunar/thunar-window.h>
-#include <thunar/thunar-shortcuts-view.h>
-#include <thunar/thunar-renamer-dialog.h>
-#include <thunar/thunar-statusbar.h>
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-compact-view.h"
+#include "thunar/thunar-details-view.h"
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-enum-types.h"
+#include "thunar/thunar-gdk-extensions.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-icon-view.h"
+#include "thunar/thunar-pango-extensions.h"
+#include "thunar/thunar-preferences-dialog.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-util.h"
+#include "thunar/thunar-window.h"
+#include "thunar/thunar-shortcuts-view.h"
+#include "thunar/thunar-renamer-dialog.h"
+#include "thunar/thunar-statusbar.h"
 
 
 
@@ -223,6 +223,20 @@ on_date_format_changed (GtkWidget *combo,
 
 
 
+static void
+on_execute_shell_scripts_changed (GtkWidget *combo,
+                                  GtkWidget *image)
+{
+  GtkComboBox *combobox = GTK_COMBO_BOX (combo);
+
+  _thunar_return_if_fail (GTK_IS_COMBO_BOX (combobox));
+  _thunar_return_if_fail (GTK_IS_WIDGET (image));
+
+  gtk_widget_set_visible (image, gtk_combo_box_get_active (combobox) != THUNAR_EXECUTE_SHELL_SCRIPT_NEVER);
+}
+
+
+
 static void
 thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
 {
@@ -251,7 +265,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   /* configure the dialog properties */
   gtk_window_set_icon_name (GTK_WINDOW (dialog), "org.xfce.thunar");
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_title (GTK_WINDOW (dialog), _("File Manager Preferences"));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Thunar Preferences"));
 
   xfce_titled_dialog_create_action_area (XFCE_TITLED_DIALOG (dialog));
 
@@ -1050,6 +1064,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Always"));
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Local Files Only"));
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Local Files On Same Devices Only"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Local Files On Idle Devices Only"));
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Never"));
   g_object_bind_property_full (G_OBJECT (dialog->preferences),
                                "misc-parallel-copy-mode",
@@ -1231,26 +1246,40 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_widget_set_margin_top (GTK_WIDGET (grid), 6);
   gtk_widget_set_margin_start (GTK_WIDGET (grid), 12);
   gtk_container_add (GTK_CONTAINER (frame), grid);
+  gtk_grid_set_column_homogeneous (GTK_GRID (grid), TRUE);
   gtk_widget_show (grid);
 
+  label = gtk_label_new_with_mnemonic (_("Execute shell scripts:"));
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, row, 1, 1);
+  gtk_widget_show (label);
+
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_grid_attach (GTK_GRID (grid), hbox, 0, row, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), hbox, 1, row, 1, 1);
   gtk_widget_show (hbox);
 
-  button = gtk_check_button_new_with_mnemonic (_("Execute shell scripts"));
+  combo = gtk_combo_box_text_new ();
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Never"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Always"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("Ask every time"));
   g_object_bind_property (G_OBJECT (dialog->preferences),
                           "misc-exec-shell-scripts-by-default",
-                          G_OBJECT (button),
+                          G_OBJECT (combo),
                           "active",
                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
+  gtk_widget_set_hexpand (combo, TRUE);
+  gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, TRUE, 0);
+  thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), combo);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
+  gtk_widget_show (combo);
 
   image = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_MENU);
   gtk_widget_set_tooltip_text (image, _("Allowing untrusted programs to run presents a security risk to your system."));
   gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
   gtk_widget_show (image);
 
+  g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (on_execute_shell_scripts_changed), image);
+
   frame = g_object_new (GTK_TYPE_FRAME, "border-width", 0, "shadow-type", GTK_SHADOW_NONE, NULL);
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
   gtk_widget_show (frame);
