@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_ERRNO_H
@@ -39,13 +39,13 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#include <thunar/thunar-file-monitor.h>
-#include <thunar/thunar-file.h>
-#include <thunar/thunar-shortcuts-model.h>
-#include <thunar/thunar-device-monitor.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-util.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-file-monitor.h"
+#include "thunar/thunar-file.h"
+#include "thunar/thunar-shortcuts-model.h"
+#include "thunar/thunar-device-monitor.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-util.h"
+#include "thunar/thunar-private.h"
 
 #define SPINNER_CYCLE_DURATION 1000
 #define SPINNER_NUM_STEPS      12
@@ -1391,8 +1391,6 @@ thunar_shortcuts_model_load (gpointer data)
 
   _thunar_return_val_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model), FALSE);
 
-THUNAR_THREADS_ENTER
-
   /* parse the bookmarks */
   thunar_util_load_bookmarks (model->bookmarks_file,
                               thunar_shortcuts_model_load_line,
@@ -1401,8 +1399,6 @@ THUNAR_THREADS_ENTER
   /* update the visibility */
   thunar_shortcuts_model_header_visibility (model);
 
-THUNAR_THREADS_LEAVE
-
   model->bookmarks_idle_id = 0;
 
   return FALSE;
@@ -1421,8 +1417,6 @@ thunar_shortcuts_model_reload (gpointer data)
 
   _thunar_return_val_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model), FALSE);
 
-THUNAR_THREADS_ENTER
-
   /* drop all existing user-defined shortcuts from the model */
   for (idx = 0, lp = model->shortcuts; lp != NULL; )
     {
@@ -1452,8 +1446,6 @@ THUNAR_THREADS_ENTER
         }
     }
 
-THUNAR_THREADS_LEAVE
-
   /* load new bookmarks */
   return thunar_shortcuts_model_load (data);
 }
