@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_LIMITS_H
@@ -30,11 +30,11 @@
 #include <locale.h>
 #endif
 
-#include <thunar/thunar-deep-count-job.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-size-label.h>
+#include "thunar/thunar-deep-count-job.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-size-label.h"
 
 
 
@@ -69,6 +69,7 @@ static void     thunar_size_label_finished              (ExoJob               *j
                                                          ThunarSizeLabel      *size_label);
 static void     thunar_size_label_status_update         (ThunarDeepCountJob   *job,
                                                          guint64               total_size,
+                                                         guint64               total_size_on_disk,
                                                          guint                 file_count,
                                                          guint                 directory_count,
                                                          guint                 unreadable_directory_count,
@@ -363,11 +364,22 @@ thunar_size_label_files_changed (ThunarSizeLabel *size_label)
       gtk_spinner_stop (GTK_SPINNER (size_label->spinner));
       gtk_widget_hide (size_label->spinner);
 
-      /* determine the size of the file */
-      size = thunar_file_get_size (THUNAR_FILE (size_label->files->data));
+      if (size_label->type == THUNAR_SIZE_LABEL_SIZE_ON_DISK)
+        {
+          /* determine the allocated size of the file */
+          size = thunar_file_get_size_on_disk (THUNAR_FILE (size_label->files->data));
+        }
+      else // if (size_label->type == THUNAR_SIZE_LABEL_SIZE)
+        {
+          /* determine the size of the file */
+          size = thunar_file_get_size (THUNAR_FILE (size_label->files->data));
+        }
 
       /* setup the new label */
-      size_string = g_format_size_full (size, size_label->file_size_binary ? G_FORMAT_SIZE_LONG_FORMAT | G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_LONG_FORMAT);
+      if (size != (guint64)-1)
+        size_string = g_format_size_full (size, size_label->file_size_binary ? G_FORMAT_SIZE_LONG_FORMAT | G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_LONG_FORMAT);
+      else
+        size_string = g_strdup_printf("%s", _("Unknown"));
       gtk_label_set_text (GTK_LABEL (size_label->label), size_string);
       g_free (size_string);
     }
@@ -413,6 +425,7 @@ thunar_size_label_finished (ExoJob          *job,
 static void
 thunar_size_label_status_update (ThunarDeepCountJob *job,
                                  guint64             total_size,
+                                 guint64             total_size_on_disk,
                                  guint               file_count,
                                  guint               directory_count,
                                  guint               unreadable_directory_count,
@@ -439,6 +452,17 @@ thunar_size_label_status_update (ThunarDeepCountJob *job,
           size_string = g_format_size_full (total_size, G_FORMAT_SIZE_LONG_FORMAT | (size_label->file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT));
           text = g_strdup_printf ("%s", size_string);
 
+          gtk_label_set_text (GTK_LABEL (size_label->label), text);
+          g_free (size_string);
+        }
+      else if (size_label->type == THUNAR_SIZE_LABEL_SIZE_ON_DISK)
+        {
+          if (total_size_on_disk != (guint64)-1)
+            size_string = g_format_size_full (total_size_on_disk, G_FORMAT_SIZE_LONG_FORMAT | (size_label->file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT));
+          else
+            size_string = g_strdup_printf("%s", _("Unknown"));
+          text = g_strdup_printf ("%s", size_string);
+
           gtk_label_set_text (GTK_LABEL (size_label->label), text);
           g_free (size_string);
         }
@@ -549,6 +573,21 @@ thunar_size_label_new (void)
 
 
 
+/**
+ * thunar_size_on_disk_label_new:
+ *
+ * Allocates a new #ThunarSizeLabel instance.
+ *
+ * Return value: the newly allocated #ThunarSizeLabel.
+ **/
+GtkWidget*
+thunar_size_on_disk_label_new (void)
+{
+  return g_object_new (THUNAR_TYPE_SIZE_LABEL, "label-type", THUNAR_SIZE_LABEL_SIZE_ON_DISK, NULL);
+}
+
+
+
 /**
  * thunar_content_label_new:
  *
