@@ -20,16 +20,16 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunar/thunar-thumbnailer-proxy.h>
-#include <thunar/thunar-marshal.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-thumbnailer.h>
+#include "thunar/thunar-thumbnailer-proxy.h"
+#include "thunar/thunar-marshal.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-thumbnailer.h"
 
 
 
@@ -147,8 +147,6 @@ static void                   thunar_thumbnailer_set_property           (GObject
 
 #define _thumbnailer_lock(thumbnailer)    g_mutex_lock (&((thumbnailer)->lock))
 #define _thumbnailer_unlock(thumbnailer)  g_mutex_unlock (&((thumbnailer)->lock))
-#define _thumbnailer_trylock(thumbnailer) g_mutex_trylock (&((thumbnailer)->lock))
-
 
 
 struct _ThunarThumbnailerClass
@@ -438,15 +436,15 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
    * processed (and awaiting to be refreshed) */
   for (lp = job->files; lp != NULL; lp = lp->next)
     {
-      /* the icon factory only loads icons for regular files, symlinks and folders */
-      if (!thunar_file_is_regular (lp->data) && !thunar_file_is_directory (lp->data) && !thunar_file_is_symlink (lp->data))
+      /* the icon factory only loads icons for regular files and folders */
+      if (!thunar_file_is_regular (lp->data) && !thunar_file_is_directory (lp->data))
         {
-          thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_NONE);
+          thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_NONE, thumbnail_size);
           continue;
         }
 
       /* get the current thumb state */
-      thumb_state = thunar_file_get_thumb_state (lp->data);
+      thumb_state = thunar_file_get_thumb_state (lp->data, thumbnail_size);
 
       if (job->lazy_checks)
         {
@@ -467,7 +465,7 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
           if (max_size > 0 && thunar_file_get_size (lp->data) > max_size)
             continue;
 
-          supported_files = g_list_prepend (supported_files, lp->data);
+          supported_files = g_list_prepend (supported_files, g_object_ref (lp->data));
           n_items++;
         }
       else
@@ -478,9 +476,9 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
 
           /* test if a thumbnail can be found */
           if (thumbnail_path != NULL && g_file_test (thumbnail_path, G_FILE_TEST_EXISTS))
-            thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_READY);
+            thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_READY, thumbnail_size);
           else
-            thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_NONE);
+            thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_NONE, thumbnail_size);
         }
     }
 
@@ -495,7 +493,7 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
       for (lp = supported_files, n = 0; lp != NULL; lp = lp->next, ++n)
         {
           /* set the thumbnail state to loading */
-          thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_LOADING);
+          thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_LOADING, thumbnail_size);
 
           /* save URI and MIME hint in the arrays */
           uris[n] = thunar_file_dup_uri (lp->data);
@@ -534,13 +532,13 @@ thunar_thumbnailer_begin_job (ThunarThumbnailer *thumbnailer,
       g_free (mime_hints);
       g_strfreev (uris);
 
-      /* free the list of supported files */
-      g_list_free (supported_files);
-
       /* free the list of files passed in */
       g_list_free_full (job->files, g_object_unref);
       job->files = NULL;
 
+      /* we need to reload the files after the job is finished */
+      job->files = supported_files;
+
       /* we assume success if we've come so far */
       success = TRUE;
     }
@@ -910,6 +908,7 @@ thunar_thumbnailer_thumbnailer_finished (GDBusProxy        *proxy,
 {
   ThunarThumbnailerJob *job;
   GSList               *lp;
+  GList                *files_to_reload = NULL;
 
   _thunar_return_if_fail (G_IS_DBUS_PROXY (proxy));
   _thunar_return_if_fail (THUNAR_IS_THUMBNAILER (thumbnailer));
@@ -934,6 +933,9 @@ thunar_thumbnailer_thumbnailer_finished (GDBusProxy        *proxy,
           /* tell everybody we're done here */
           g_signal_emit (G_OBJECT (thumbnailer), thumbnailer_signals[REQUEST_FINISHED], 0, job->request);
 
+          for (GList *file_lp = job->files; file_lp != NULL; file_lp = file_lp->next)
+            files_to_reload = g_list_append (files_to_reload, g_object_ref (file_lp->data));
+
           /* remove job from the list */
           thumbnailer->jobs = g_slist_delete_link (thumbnailer->jobs, lp);
 
@@ -943,6 +945,13 @@ thunar_thumbnailer_thumbnailer_finished (GDBusProxy        *proxy,
     }
 
   _thumbnailer_unlock (thumbnailer);
+
+  /* Do the reload outside of teh critical section in order to prevent a deadlock */
+  for (GList *file_lp = files_to_reload; file_lp != NULL; file_lp = file_lp->next)
+    thunar_file_reload (THUNAR_FILE (file_lp->data));
+  
+  if (files_to_reload != NULL)
+    g_list_free_full (files_to_reload, g_object_unref);
 }
 
 
@@ -1031,13 +1040,13 @@ thunar_thumbnailer_idle_func (gpointer user_data)
             {
               /* set thumbnail state to none unless the thumbnail has already been created.
                * This is to prevent race conditions with the other idle functions */
-              if (thunar_file_get_thumb_state (file) != THUNAR_FILE_THUMB_STATE_READY)
-                thunar_file_set_thumb_state (file, THUNAR_FILE_THUMB_STATE_NONE);
+              if (thunar_file_get_thumb_state (file, idle->thumbnailer->thumbnail_size) != THUNAR_FILE_THUMB_STATE_READY)
+                thunar_file_set_thumb_state (file, THUNAR_FILE_THUMB_STATE_NONE, idle->thumbnailer->thumbnail_size);
             }
           else if (idle->type == THUNAR_THUMBNAILER_IDLE_READY)
             {
               /* set thumbnail state to ready - we now have a thumbnail */
-              thunar_file_set_thumb_state (file, THUNAR_FILE_THUMB_STATE_READY);
+              thunar_file_set_thumb_state (file, THUNAR_FILE_THUMB_STATE_READY, idle->thumbnailer->thumbnail_size);
             }
           else
             {
