@@ -18,42 +18,45 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <exo/exo.h>
 
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <thunar/thunar-abstract-dialog.h>
-#include <thunar/thunar-toolbar-editor.h>
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-pango-extensions.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-application.h>
+#include "thunar/thunar-abstract-dialog.h"
+#include "thunar/thunar-toolbar-editor.h"
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-pango-extensions.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-application.h"
 
 
 
-static void thunar_toolbar_editor_finalize                           (GObject                  *object);
-static void thunar_toolbar_editor_help_clicked                       (ThunarToolbarEditor      *toolbar_editor,
-                                                                      GtkWidget                *button);
-static void thunar_toolbar_editor_swap_toolbar_items_for_all_windows (ThunarToolbarEditor      *editor,
-                                                                      GtkTreeIter              *item1,
-                                                                      GtkTreeIter              *item2);
-static void thunar_toolbar_editor_move_down                          (ThunarToolbarEditor      *toolbar_editor,
-                                                                      GtkWidget                *button);
-static void thunar_toolbar_editor_move_up                            (ThunarToolbarEditor      *toolbar_editor,
-                                                                      GtkWidget                *button);
-static void thunar_toolbar_editor_toggle_visibility                  (ThunarToolbarEditor      *toolbar_editor,
-                                                                      const gchar              *path_string,
-                                                                      GtkCellRendererToggle    *cell_renderer);
-static void thunar_toolbar_editor_update_buttons                     (ThunarToolbarEditor      *toolbar_editor);
-static void thunar_toolbar_editor_use_defaults                       (ThunarToolbarEditor      *toolbar_editor,
-                                                                      GtkWidget                *button);
-static void thunar_toolbar_editor_save_model                         (ThunarToolbarEditor      *toolbar_editor);
-static void thunar_toolbar_editor_populate_model                     (ThunarToolbarEditor      *toolbar_editor);
+static void     thunar_toolbar_editor_finalize                           (GObject                  *object);
+static gboolean thunar_toolbar_editor_visible_func                       (GtkTreeModel             *model,
+                                                                          GtkTreeIter              *iter,
+                                                                          gpointer                  data);
+static void     thunar_toolbar_editor_help_clicked                       (ThunarToolbarEditor      *toolbar_editor,
+                                                                          GtkWidget                *button);
+static void     thunar_toolbar_editor_swap_toolbar_items_for_all_windows (GtkListStore             *model,
+                                                                          GtkTreeIter              *item1,
+                                                                          GtkTreeIter              *item2);
+static void     thunar_toolbar_editor_move_down                          (ThunarToolbarEditor      *toolbar_editor,
+                                                                          GtkWidget                *button);
+static void     thunar_toolbar_editor_move_up                            (ThunarToolbarEditor      *toolbar_editor,
+                                                                          GtkWidget                *button);
+static void     thunar_toolbar_editor_toggle_visibility                  (ThunarToolbarEditor      *toolbar_editor,
+                                                                          const gchar              *path_string,
+                                                                          GtkCellRendererToggle    *cell_renderer);
+static void     thunar_toolbar_editor_update_buttons                     (ThunarToolbarEditor      *toolbar_editor);
+static void     thunar_toolbar_editor_use_defaults                       (ThunarToolbarEditor      *toolbar_editor,
+                                                                          GtkWidget                *button);
+static void     thunar_toolbar_editor_save_model                         (ThunarToolbarEditor      *toolbar_editor);
+static void     thunar_toolbar_editor_populate_model                     (ThunarToolbarEditor      *toolbar_editor);
 
 
 
@@ -69,6 +72,7 @@ struct _ThunarToolbarEditor
   ThunarPreferences *preferences;
 
   GtkListStore      *model;
+  GtkTreeModel      *filter;
 
   /* Used to get information about the toolbar items, it is used as a reference. All the toolbars are changed, not just this one. */
   GtkWidget         *toolbar;
@@ -99,15 +103,13 @@ static void
 thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
 {
   GtkCellRenderer        *renderer;
-  GtkWidget              *separator;
   GtkWidget              *button;
-  GtkWidget              *frame;
   GtkWidget              *image;
   GtkWidget              *label;
-  GtkWidget              *grid;
   GtkWidget              *vbox;
+  GtkWidget              *hbox;
+  GtkWidget              *vbox_buttons;
   GtkWidget              *swin;
-  gint                    row = 0;
 
   /* grab a reference on the preferences */
   toolbar_editor->preferences = thunar_preferences_get ();
@@ -118,7 +120,8 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   /* setup the dialog */
   gtk_dialog_add_button (GTK_DIALOG (toolbar_editor), _("_Close"), GTK_RESPONSE_CLOSE);
   gtk_dialog_set_default_response (GTK_DIALOG (toolbar_editor), GTK_RESPONSE_CLOSE);
-  gtk_window_set_resizable (GTK_WINDOW (toolbar_editor), FALSE);
+  gtk_window_set_default_size (GTK_WINDOW (toolbar_editor), 510, 490);
+  gtk_window_set_resizable (GTK_WINDOW (toolbar_editor), TRUE);
   gtk_window_set_title (GTK_WINDOW (toolbar_editor), _("Configure the Toolbar"));
 
   /* add the "Help" button */
@@ -129,30 +132,21 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   gtk_widget_show (button);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (toolbar_editor))), vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
-  frame = g_object_new (GTK_TYPE_FRAME, "border-width", 0, "shadow-type", GTK_SHADOW_NONE, NULL);
-  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
-  gtk_widget_show (frame);
-
-  grid = gtk_grid_new ();
-  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
-  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
-  gtk_container_set_border_width (GTK_CONTAINER (grid), 12);
-  gtk_container_add (GTK_CONTAINER (frame), grid);
-  gtk_widget_show (grid);
-
   /* create the top label for the toolbar editor dialog */
   label = gtk_label_new (_("Configure the order and visibility of toolbar items."));
   gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
   gtk_widget_set_hexpand (label, TRUE);
-  gtk_grid_attach (GTK_GRID (grid), label, 0, row, 2, 1);
+  gtk_widget_set_vexpand (label, FALSE);
+  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
-  /* next row */
-  row++;
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 6);
+  gtk_widget_show (hbox);
 
   /* create the scrolled window for the tree view */
   swin = gtk_scrolled_window_new (NULL, NULL);
@@ -160,13 +154,19 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin), GTK_SHADOW_IN);
   gtk_widget_set_hexpand (swin, TRUE);
   gtk_widget_set_vexpand (swin, TRUE);
-  gtk_grid_attach (GTK_GRID (grid), swin, 0, row, 1, 6);
+  gtk_box_pack_start (GTK_BOX (hbox), swin, TRUE, TRUE, 0);
   gtk_widget_show (swin);
 
+  /* create a filter from the shared column model */
+  toolbar_editor->filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (toolbar_editor->model), NULL);
+  gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (toolbar_editor->filter),
+                                          (GtkTreeModelFilterVisibleFunc) thunar_toolbar_editor_visible_func,
+                                          NULL, NULL);
+
   /* create the tree view */
-  toolbar_editor->tree_view = gtk_tree_view_new ();
+  toolbar_editor->tree_view = gtk_tree_view_new_with_model (toolbar_editor->filter);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (toolbar_editor->tree_view), FALSE);
-  gtk_tree_view_set_model (GTK_TREE_VIEW (toolbar_editor->tree_view), GTK_TREE_MODEL (toolbar_editor->model));
+  gtk_tree_view_set_search_column (GTK_TREE_VIEW (toolbar_editor->tree_view), 2);
   gtk_container_add (GTK_CONTAINER (swin), toolbar_editor->tree_view);
   gtk_widget_show (toolbar_editor->tree_view);
 
@@ -198,10 +198,16 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
                                                "text", 2,
                                                NULL);
 
+  /* Create the buttons vbox container */
+  vbox_buttons = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox_buttons, FALSE, FALSE, 0);
+  gtk_widget_set_vexpand (vbox_buttons, FALSE);
+  gtk_widget_show (vbox_buttons);
+
   /* create the "Move Up" button */
   toolbar_editor->up_button = gtk_button_new_with_mnemonic (_("Move _Up"));
   g_signal_connect_swapped (G_OBJECT (toolbar_editor->up_button), "clicked", G_CALLBACK (thunar_toolbar_editor_move_up), toolbar_editor);
-  gtk_grid_attach (GTK_GRID (grid), toolbar_editor->up_button, 1, row, 1, 1);
+  gtk_box_pack_start (GTK_BOX (vbox_buttons), toolbar_editor->up_button, FALSE, FALSE, 0);
   gtk_widget_show (toolbar_editor->up_button);
 
   image = gtk_image_new_from_icon_name ("go-up-symbolic", GTK_ICON_SIZE_BUTTON);
@@ -209,13 +215,10 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   gtk_button_set_image (GTK_BUTTON (toolbar_editor->up_button), image);
   gtk_widget_show (image);
 
-  /* next row */
-  row++;
-
   /* create the "Move Down" button */
   toolbar_editor->down_button = gtk_button_new_with_mnemonic (_("Move Dow_n"));
   g_signal_connect_swapped (G_OBJECT (toolbar_editor->down_button), "clicked", G_CALLBACK (thunar_toolbar_editor_move_down), toolbar_editor);
-  gtk_grid_attach (GTK_GRID (grid), toolbar_editor->down_button, 1, row, 1, 1);
+  gtk_box_pack_start (GTK_BOX (vbox_buttons), toolbar_editor->down_button, FALSE, FALSE, 0);
   gtk_widget_show (toolbar_editor->down_button);
 
   image = gtk_image_new_from_icon_name ("go-down-symbolic", GTK_ICON_SIZE_BUTTON);
@@ -223,27 +226,12 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   gtk_button_set_image (GTK_BUTTON (toolbar_editor->down_button), image);
   gtk_widget_show (image);
 
-  /* next row */
-  row++;
-
-  /* create the horiz separator */
-  separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-  gtk_grid_attach (GTK_GRID (grid), separator, 1, row, 1, 1);
-  gtk_widget_show (separator);
-
-  /* next row */
-  row++;
-
   /* create the "Use Default" button */
   button = gtk_button_new_with_mnemonic (_("De_fault Order"));
   g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_toolbar_editor_use_defaults), toolbar_editor);
-  gtk_grid_attach (GTK_GRID (grid), button, 1, row, 1, 1);
+  gtk_box_pack_start (GTK_BOX (vbox_buttons), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
 
-  frame = g_object_new (GTK_TYPE_FRAME, "border-width", 0, "shadow-type", GTK_SHADOW_NONE, NULL);
-  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, TRUE, 0);
-  gtk_widget_show (frame);
-
   /* grab focus to the tree view */
   gtk_widget_grab_focus (toolbar_editor->tree_view);
 }
@@ -269,6 +257,25 @@ thunar_toolbar_editor_finalize (GObject *object)
 
 
 
+static gboolean
+thunar_toolbar_editor_visible_func (GtkTreeModel *model,
+                                    GtkTreeIter  *iter,
+                                    gpointer     data)
+{
+  gboolean visible = TRUE;
+  gint     order;
+
+  gtk_tree_model_get (model, iter, 3, &order, -1);
+
+   /* The item with order 0 is always THUNAR_WINDOW_ACTION_VIEW_MENUBAR which we want to hide */
+  if (order == 0)
+    visible = FALSE;
+
+  return visible;
+}
+
+
+
 static void
 thunar_toolbar_editor_help_clicked (ThunarToolbarEditor *toolbar_editor,
                                     GtkWidget          *button)
@@ -286,23 +293,26 @@ thunar_toolbar_editor_help_clicked (ThunarToolbarEditor *toolbar_editor,
 
 
 static void
-thunar_toolbar_editor_swap_toolbar_items_for_all_windows (ThunarToolbarEditor *editor,
-                                                          GtkTreeIter         *item1,
-                                                          GtkTreeIter         *item2)
+thunar_toolbar_editor_swap_toolbar_items_for_all_windows (GtkListStore *model,
+                                                          GtkTreeIter  *item1,
+                                                          GtkTreeIter  *item2)
 {
-  GList            *windows;
-  GtkTreePath      *path1;
-  GtkTreePath      *path2;
+  ThunarApplication *application;
+  GList             *windows;
+  GtkTreePath       *path1;
+  GtkTreePath       *path2;
 
-  path1 = gtk_tree_model_get_path (GTK_TREE_MODEL (editor->model), item1);
-  path2 = gtk_tree_model_get_path (GTK_TREE_MODEL (editor->model), item2);
+  path1 = gtk_tree_model_get_path (GTK_TREE_MODEL (model), item1);
+  path2 = gtk_tree_model_get_path (GTK_TREE_MODEL (model), item2);
 
-  windows = thunar_application_get_windows (thunar_application_get ());
+  application = thunar_application_get ();
+  windows = thunar_application_get_windows (application);
   for (GList *lp = windows; lp != NULL; lp = lp->next)
     {
       ThunarWindow *window = lp->data;
       thunar_window_toolbar_swap_items (window, gtk_tree_path_get_indices (path1)[0], gtk_tree_path_get_indices (path2)[0]);
     }
+  g_object_unref (application);
   g_list_free (windows);
 
   gtk_tree_path_free (path1);
@@ -317,8 +327,11 @@ thunar_toolbar_editor_move_down (ThunarToolbarEditor *toolbar_editor,
 {
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
+  GtkTreeModel     *childModel;
   GtkTreeIter       iter1;
   GtkTreeIter       iter2;
+  GtkTreeIter       childIter1;
+  GtkTreeIter       childIter2;
 
   _thunar_return_if_fail (THUNAR_IS_TOOLBAR_EDITOR (toolbar_editor));
   _thunar_return_if_fail (GTK_IS_BUTTON (button));
@@ -330,8 +343,13 @@ thunar_toolbar_editor_move_down (ThunarToolbarEditor *toolbar_editor,
   if (gtk_tree_model_iter_next (model, &iter2) == FALSE)
     return;
 
-  gtk_list_store_swap (GTK_LIST_STORE (model), &iter1, &iter2);
-  thunar_toolbar_editor_swap_toolbar_items_for_all_windows (toolbar_editor, &iter1, &iter2);
+  /* tree view's model is made from GTK_TREE_MODEL_FILTER, hence fetching child model and child iter's */
+  gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter1, &iter1);
+  gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter2, &iter2);
+  childModel = gtk_tree_model_filter_get_model(GTK_TREE_MODEL_FILTER (model));
+
+  gtk_list_store_swap (GTK_LIST_STORE (childModel), &childIter1, &childIter2);
+  thunar_toolbar_editor_swap_toolbar_items_for_all_windows (GTK_LIST_STORE (childModel), &childIter1, &childIter2);
 }
 
 
@@ -342,8 +360,11 @@ thunar_toolbar_editor_move_up (ThunarToolbarEditor *toolbar_editor,
 {
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
+  GtkTreeModel     *childModel;
   GtkTreeIter       iter1;
   GtkTreeIter       iter2;
+  GtkTreeIter       childIter1;
+  GtkTreeIter       childIter2;
 
   _thunar_return_if_fail (THUNAR_IS_TOOLBAR_EDITOR (toolbar_editor));
   _thunar_return_if_fail (GTK_IS_BUTTON (button));
@@ -355,8 +376,13 @@ thunar_toolbar_editor_move_up (ThunarToolbarEditor *toolbar_editor,
   if (gtk_tree_model_iter_previous (model, &iter2) == FALSE)
     return;
 
-  gtk_list_store_swap (GTK_LIST_STORE (model), &iter1, &iter2);
-  thunar_toolbar_editor_swap_toolbar_items_for_all_windows (toolbar_editor, &iter1, &iter2);
+  /* tree view's model is made from GTK_TREE_MODEL_FILTER, hence fetching child model and child iter's */
+  gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter1, &iter1);
+  gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter2, &iter2);
+  childModel = gtk_tree_model_filter_get_model(GTK_TREE_MODEL_FILTER (model));
+
+  gtk_list_store_swap (GTK_LIST_STORE (childModel), &childIter1, &childIter2);
+  thunar_toolbar_editor_swap_toolbar_items_for_all_windows (GTK_LIST_STORE (childModel), &childIter1, &childIter2);
 }
 
 
@@ -366,10 +392,12 @@ thunar_toolbar_editor_toggle_visibility (ThunarToolbarEditor    *toolbar_editor,
                                          const gchar            *path_string,
                                          GtkCellRendererToggle  *cell_renderer)
 {
-  GtkTreePath *path;
-  GtkTreeIter  iter;
-  gboolean     visible;
-  GList       *windows;
+  ThunarApplication *application;
+  GList             *windows;
+  GtkTreePath       *path;
+  GtkTreePath       *child_path;
+  GtkTreeIter        iter;
+  gboolean           visible;
 
   _thunar_return_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (cell_renderer));
   _thunar_return_if_fail (THUNAR_IS_TOOLBAR_EDITOR (toolbar_editor));
@@ -377,21 +405,33 @@ thunar_toolbar_editor_toggle_visibility (ThunarToolbarEditor    *toolbar_editor,
 
   /* determine the tree path for the string */
   path = gtk_tree_path_new_from_string (path_string);
-  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (toolbar_editor->model), &iter, path))
+
+  child_path = gtk_tree_model_filter_convert_path_to_child_path (GTK_TREE_MODEL_FILTER (toolbar_editor->filter), path);
+
+  if (child_path == NULL)
+    {
+      gtk_tree_path_free (path);
+      return;
+    }
+
+  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (toolbar_editor->model), &iter, child_path))
     {
       gtk_tree_model_get (GTK_TREE_MODEL (toolbar_editor->model), &iter, 0, &visible, -1);
       gtk_list_store_set (toolbar_editor->model, &iter, 0, !visible, -1);
     }
 
-  windows = thunar_application_get_windows (thunar_application_get ());
+  application = thunar_application_get ();
+  windows = thunar_application_get_windows (application);
   for (GList *lp = windows; lp != NULL; lp = lp->next)
     {
       ThunarWindow *window = lp->data;
-      thunar_window_toolbar_toggle_item_visibility (window, gtk_tree_path_get_indices (path)[0]);
+      thunar_window_toolbar_toggle_item_visibility (window, gtk_tree_path_get_indices (child_path)[0]);
     }
-
+  g_object_unref (application);
   g_list_free (windows);
+
   gtk_tree_path_free (path);
+  gtk_tree_path_free (child_path);
 }
 
 
@@ -475,7 +515,7 @@ thunar_toolbar_editor_use_defaults (ThunarToolbarEditor *toolbar_editor,
               gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (toolbar_editor->model), &iter_i, path_i);
               gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (toolbar_editor->model), &iter_j, path_j);
               gtk_list_store_swap (GTK_LIST_STORE (toolbar_editor->model), &iter_i, &iter_j);
-              thunar_toolbar_editor_swap_toolbar_items_for_all_windows (toolbar_editor, &iter_i, &iter_j);
+              thunar_toolbar_editor_swap_toolbar_items_for_all_windows (toolbar_editor->model, &iter_i, &iter_j);
 
               y = current_order[i];
               current_order[i] = target_order[i];
@@ -573,6 +613,7 @@ thunar_toolbar_editor_populate_model (ThunarToolbarEditor *toolbar_editor)
       order = g_object_get_data (G_OBJECT (item), "default-order");
 
       widget = gtk_label_new_with_mnemonic (label_with_mnemonic);
+      g_object_ref_sink (widget);
 
       gtk_list_store_append (toolbar_editor->model, &iter);
       gtk_list_store_set (toolbar_editor->model, &iter,
@@ -582,7 +623,7 @@ thunar_toolbar_editor_populate_model (ThunarToolbarEditor *toolbar_editor)
                           3, *order,
                           -1);
 
-      gtk_widget_destroy (widget);
+      g_object_unref (widget);
     }
 
   /* connect after model has been fully initialized */
@@ -596,7 +637,7 @@ thunar_toolbar_editor_populate_model (ThunarToolbarEditor *toolbar_editor)
   g_signal_connect_swapped (G_OBJECT (selection), "changed", G_CALLBACK (thunar_toolbar_editor_update_buttons), toolbar_editor);
 
   /* select the first item */
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (toolbar_editor->model), &iter))
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (toolbar_editor->filter), &iter))
     gtk_tree_selection_select_iter (selection, &iter);
 
   g_list_free (toolbar_items);
