@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_MEMORY_H
@@ -29,15 +29,15 @@
 #include <string.h>
 #endif
 
-#include <thunar/thunar-file-monitor.h>
-#include <thunar/thunar-folder.h>
-#include <thunar/thunar-gio-extensions.h>
-#include <thunar/thunar-pango-extensions.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-tree-model.h>
-#include <thunar/thunar-device-monitor.h>
-#include <thunar/thunar-util.h>
+#include "thunar/thunar-file-monitor.h"
+#include "thunar/thunar-folder.h"
+#include "thunar/thunar-gio-extensions.h"
+#include "thunar/thunar-pango-extensions.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-tree-model.h"
+#include "thunar/thunar-device-monitor.h"
+#include "thunar/thunar-util.h"
 
 
 
@@ -940,14 +940,10 @@ thunar_tree_model_cleanup_idle (gpointer user_data)
 {
   ThunarTreeModel *model = THUNAR_TREE_MODEL (user_data);
 
-THUNAR_THREADS_ENTER
-
   /* walk through the tree and release all the nodes with a ref count of 0 */
   g_node_traverse (model->root, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
                    thunar_tree_model_node_traverse_cleanup, model);
 
-THUNAR_THREADS_LEAVE
-
   return FALSE;
 }
 
@@ -1439,8 +1435,6 @@ thunar_tree_model_item_load_idle (gpointer user_data)
       _thunar_return_val_if_fail (node->children == NULL || G_NODE_HAS_DUMMY (node), FALSE);
 #endif
 
-THUNAR_THREADS_ENTER
-
   /* check if we don't have a file yet and this is a mounted volume */
   if (item->file == NULL && item->device != NULL && thunar_device_is_mounted (item->device))
     {
@@ -1468,7 +1462,10 @@ THUNAR_THREADS_ENTER
           /* load the initial set of files (if any) */
           files = thunar_folder_get_files (item->folder);
           if (G_UNLIKELY (files != NULL))
-            thunar_tree_model_item_files_added (item, files, item->folder);
+            {
+              thunar_tree_model_item_files_added (item, files, item->folder);
+              g_list_free (files);
+            }
 
           /* notify for "loading" if already loaded */
           if (!thunar_folder_get_loading (item->folder))
@@ -1476,8 +1473,6 @@ THUNAR_THREADS_ENTER
         }
     }
 
-THUNAR_THREADS_LEAVE
-
   return FALSE;
 }
 
