@@ -18,13 +18,13 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-view.h>
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-view.h"
 
 
 
