@@ -20,9 +20,9 @@
 #ifndef __THUNAR_WINDOW_H__
 #define __THUNAR_WINDOW_H__
 
-#include <thunar/thunar-action-manager.h>
-#include <thunar/thunar-enum-types.h>
-#include <thunar/thunar-folder.h>
+#include "thunar/thunar-action-manager.h"
+#include "thunar/thunar-enum-types.h"
+#include "thunar/thunar-folder.h"
 
 #include <libxfce4ui/libxfce4ui.h>
 
@@ -56,6 +56,7 @@ typedef enum
   THUNAR_WINDOW_ACTION_RELOAD,
   THUNAR_WINDOW_ACTION_RELOAD_ALT,
   THUNAR_WINDOW_ACTION_VIEW_SPLIT,
+  THUNAR_WINDOW_ACTION_SWITCH_FOCUSED_SPLIT_VIEW_PANE,
   THUNAR_WINDOW_ACTION_VIEW_LOCATION_SELECTOR_MENU,
   THUNAR_WINDOW_ACTION_VIEW_LOCATION_SELECTOR_ENTRY,
   THUNAR_WINDOW_ACTION_VIEW_LOCATION_SELECTOR_BUTTONS,
